"""pyMOOSE

Python bindings of MOOSE simulator.

References:
-----------

- `Documentation https://moose.readthedocs.io/en/latest/`
- `Development https://github.com/BhallaLab/moose-core`

"""

# Notes
# -----
# Use these guidelines for docstring: https://numpydoc.readthedocs.io/en/latest/format.html

import sys
import pydoc
import os

import moose._moose as _moose
from moose import model_utils

__moose_classes__ = {}


class melement(_moose.ObjId):
    """Base class for all moose classes.
    """

    __type__ = "UNKNOWN"
    __doc__ = ""

    def __init__(self, x, ndata=1, **kwargs):
        obj = _moose.__create__(self.__type__, x, ndata)
        if sys.version_info.major > 2:
            super().__init__(obj)
            for k, v in kwargs.items():
                super().setField(k, v)
        else:
            # Support for dead python2.
            super(melement, self).__init__(obj)
            for k, v in kwargs.items():
                super(melement, self).setField(k, v)


def __to_melement(obj):
    global __moose_classes__
    mc = __moose_classes__[obj.type](obj)
    return mc


# Create MOOSE classes from available Cinfos.
for p in _moose.wildcardFind("/##[TYPE=Cinfo]"):
    if sys.version_info.major > 2:
        cls = type(
            p.name,
            (melement,),
            {"__type__": p.name, "__doc__": _moose.__generatedoc__(p.name)},
        )
    else:
        # Python2.
        cls = type(
            str(p.name),
            (melement,),
            {"__type__": p.name, "__doc__": _moose.__generatedoc__(p.name)},
        )
    setattr(_moose, cls.__name__, cls)
    __moose_classes__[cls.__name__] = cls


# Import all attributes to global namespace. We must do it here after adding
# class types to _moose.
from moose._moose import *

def version():
    """Reutrns moose version string."""
    return _moose.__version__

__version__ = version()

def version_info():
    """Return detailed version information.

    >>> moose.version_info()
    {'build_datetime': 'Friday Fri Apr 17 22:13:00 2020',
     'compiler_string': 'GNU,/usr/bin/c++,7.5.0',
     'major': '3',
     'minor': '3',
     'patch': '1'}
    """
    return _moose.version_info()


def about():
    """general information about pyMOOSE.

    Returns
    -------
    A dict

    Example
    -------
    >>> moose.about()
    {'path': '/home/dilawars/Work/GITLAB/moose-core/_build/python/moose',
     'version': '4.0.0.dev20200417',
     'docs': 'https://moose.readthedocs.io/en/latest/',
     'development': 'https://github.com/BhallaLab/moose-core'}
    """
    return dict(
        path=os.path.dirname(__file__),
        version=_moose.__version__,
        docs="https://moose.readthedocs.io/en/latest/",
        development="https://github.com/BhallaLab/moose-core",
    )


def wildcardFind(pattern):
    """Find objects using wildcard pattern

    Parameters
    ----------
    pattern: str
       Wildcard (see note below)

    .. note:: Wildcard

    MOOSE allows wildcard expressions of the form
    {PATH}/{WILDCARD}[{CONDITION}].

    {PATH} is valid path in the element tree, {WILDCARD} can be
    # or ##. # causes the search to be restricted to the children
    of the element specified by {PATH}. ## makes the search to
    recursively go through all the descendants of the {PATH} element.

    {CONDITION} can be:

    - TYPE={CLASSNAME}: an element satisfies this condition if it is of
      class {CLASSNAME}.
    - ISA={CLASSNAME}: alias for TYPE={CLASSNAME}
    - CLASS={CLASSNAME}: alias for TYPE={CLASSNAME}
    - FIELD({FIELDNAME}){OPERATOR}{VALUE} : compare field {FIELDNAME} with
      {VALUE} by {OPERATOR} where {OPERATOR} is a comparison
      operator (=, !=, >, <, >=, <=).

    Returns
    -------
    list
        A list of found MOOSE objects

    Examples
    --------
    Following returns a list of all the objects under /mymodel whose Vm field
    is >= -65.

    >>> moose.wildcardFind("/mymodel/##[FIELD(Vm)>=-65]")
    """
    return [__to_melement(x) for x in _moose.wildcardFind(pattern)]


def connect(src, srcfield, dest, destfield, msgtype="Single"):
    """Create a message between `srcfield` on `src` object to
     `destfield` on `dest` object.

     This function is used mainly, to say, connect two entities, and
     to denote what kind of give-and-take relationship they share.
     It enables the 'destfield' (of the 'destobj') to acquire the
     data, from 'srcfield'(of the 'src').

     Parameters
     ----------
     src : element/vec/string
         the source object (or its path) the one that provides information.
     srcfield : str
         source field on self (type of the information).
     destobj : element
         Destination object to connect to (The one that need to get
         information).
     destfield : str
         field to connect to on `destobj`
     msgtype : str
         type of the message. It ca be one of the following (default Single).
         - Single
         - OneToAll
         - AllToOne
         - OneToOne
         - Reduce
         - Sparse

     Returns
     -------
     msgmanager: melement
         message-manager for the newly created message.

     Note
     -----
     Alternatively, one can also use the following form::

     >>> src.connect(srcfield, dest, destfield, msgtype)


     Examples
     --------
     Connect the output of a pulse generator to the input of a spike generator::

     >>> pulsegen = moose.PulseGen('pulsegen')
     >>> spikegen = moose.SpikeGen('spikegen')
     >>> moose.connect(pulsegen, 'output', spikegen, 'Vm')
     Or,
     >>> pulsegen.connect('output', spikegen, 'Vm')
    """
    src = _moose.element(src)
    dest = _moose.element(dest)
    return src.connect(srcfield, dest, destfield, msgtype)


def delete(arg):
    """Delete the underlying moose object(s). 

    Parameters
    ----------
    arg : vec/str/melement
        path of the object to be deleted.

    Returns
    -------
    None, Raises ValueError if given path/object does not exists.
    """
    _moose.delete(arg)


def element(arg):
    """Convert a path or an object to the appropriate builtin moose class instance

    Parameters
    ----------
    arg : str/vec/moose object
        path of the moose element to be converted or another element (possibly
        available as a superclass instance).

    Returns
    -------
    melement
        MOOSE element (object) corresponding to the `arg` converted to write
        subclass.
    """
    if isinstance(arg, str):
        assert _moose.exists(arg), "Path '%s' does not exists." % arg
    return _moose.element(arg)


def exists(path):
    """Returns True if an object with given path already exists."""
    return _moose.exists(path)


def getCwe():
    """Return current working elemement.

    See also
    --------
    moose.setCwe
    """
    return _moose.getCwe()


def getField(classname, fieldname):
    """Get specified field of specified class."""
    return _moose.getField(classname, fieldname)


def getFieldDict(classname, finfoType=""):
    """Get dictionary of field names and types for specified class.

    Parameters
    ----------
    className : str
        MOOSE class to find the fields of.
    finfoType : str (default '')
        Finfo type of the fields to find. If empty or not specified, allfields
        will be retrieved.

    Returns
    -------
    dict
        field names and their respective types as key-value pair. 

    Notes
    -----
    This behaviour is different from `getFieldNames` where only `valueFinfo`s
    are returned when `finfoType` remains unspecified.

    Examples
    --------
    List all the source fields on class Neutral

    >>> moose.getFieldDict('Neutral', 'srcFinfo')
       {'childMsg': 'int'}
    """
    return _moose.getFieldDict(classname, finfoType)


def getFieldNames(elem, fieldtype="*"):
    """Get a tuple containing name of fields of a given fieldtype. If
    fieldtype is set to '*', all fields are returned.

    Parameters
    ----------
    elem : string, obj
        Name of the class or a moose.element to look up.
    fieldtype : string
        The kind of field. Possible values are:
        -  'valueFinfo' or 'value'
        -  'srcFinfo' or 'src'
        -  'destFinfo' or 'dest'
        -  'lookupFinfo' or 'lookup'
        -  'fieldElementFinfo' or 'fieldElement'

    Returns
    -------
    list
        Names of the fields of type `finfoType` in class `className`.
    """
    clsname = elem if isinstance(elem, str) else elem.className
    return _moose.getFieldNames(clsname, fieldtype)


def isRunning():
    """True if the simulation is currently running."""
    return _moose.isRunning()


def move(src, dest):
    """Move a moose element `src` to destination"""
    return _moose.move(src, dest)


def reinit():
    """Reinitialize simulation.

    This function (re)initializes moose simulation. It must be called before
    you start the simulation (see moose.start). If you want to continue
    simulation after you have called moose.reinit() and moose.start(), you must
    NOT call moose.reinit() again. Calling moose.reinit() again will take the
    system back to initial setting (like clear out all data recording tables,
    set state variables to their initial values, etc.
    """
    _moose.reinit()


def start(runtime, notify=False):
    """Run simulation for `t` time. Advances the simulator clock by `t` time. If
    'notify = True', a message is written to terminal whenever 10\% of
    simulation time is over. \

    After setting up a simulation, YOU MUST CALL MOOSE.REINIT() before CALLING
    MOOSE.START() TO EXECUTE THE SIMULATION. Otherwise, the simulator behaviour
    will be undefined. Once moose.reinit() has been called, you can call
    `moose.start(t)` as many time as you like. This will continue the
    simulation from the last state for `t` time.

    Parameters
    ----------
    t : float
        duration of simulation.
    notify: bool
        default False. If True, notify user whenever 10\% of simultion is over.

    Returns
    -------
        None

    See also
    --------
    moose.reinit : (Re)initialize simulation
    """
    _moose.start(runtime, notify)


def stop():
    """Stop simulation"""
    _moose.stop()


def setCwe(arg):
    """Set the current working element.

    Parameters
    ----------
    arg: str, melement, vec
        moose element or path to be set as cwe.

    See also
    --------
    getCwe
    """
    _moose.setCwe(arg)

def ce(arg):
    """Alias for setCwe"""
    _moose.setCwe(arg)

def useClock(tick, path, fn):
    """Schedule `fn` function of every object that matches `path` on tick no
    `tick`. Usually you don't have to use it.

    (FIXME: Needs update) The sequence of clockticks with the same dt is
    according to their number.  This is utilized for controlling the order of
    updates in various objects where it matters.  The following convention
    should be observed when assigning clockticks to various components of a
    model: Clock ticks 0-3 are for electrical (biophysical) components, 4 and 5
    are for chemical kinetics, 6 and 7 are for lookup tables and stimulus, 8
    and 9 are for recording tables.

    Parameters
    ----------
    tick : int
        tick number on which the targets should be scheduled.
    path : str
        path of the target element(s). This can be a wildcard also.
    fn : str
        name of the function to be called on each tick. Commonly `process`.

    Examples
    --------
    In multi-compartmental neuron model a compartment's membrane potential (Vm)
    is dependent on its neighbours' membrane potential. Thus it must get the
    neighbour's present Vm before computing its own Vm in next time step.  This
    ordering is achieved by scheduling the `init` function, which communicates
    membrane potential, on tick 0 and `process` function on tick 1.

    >>> moose.useClock(0, '/model/compartment_1', 'init')
    >>> moose.useClock(1, '/model/compartment_1', 'process'));
    """
    _moose.useClock(tick, path, fn)


def setClock(clockid, dt):
    """set the ticking interval of `tick` to `dt`.

    A tick with interval `dt` will call the functions scheduled on that tick
    every `dt` timestep.

    Parameters
    ----------
    tick : int
        tick number
    dt : double
        ticking interval

    """
    _moose.setClock(clockid, dt)


def loadModel(filename, modelpath, solverclass="gsl"):
    """loadModel: Load model (genesis/cspace) from a file to a specified path.

    Parameters
    ----------
    filename: str
        model description file.
    modelpath: str
        moose path for the top level element of the model to be created.
    method: str
        solver type to be used for simulating the model.
        TODO: Link to detailed description of solvers?

    Returns
    -------
    melement
        moose.element if succcessful else None.

    See also
    --------
    moose.readNML2
    moose.writeNML2 (NotImplemented)
    moose.readSBML
    moose.writeSBML
    """
    return model_utils.mooseReadKkitGenesis(filename, modelpath, solverclass)


def copy(src, dest, name="", n=1, toGlobal=False, copyExtMsg=False):
    """Make copies of a moose object.

    Parameters
    ----------
    src : vec, element or str
        source object.
    dest : vec, element or str
        Destination object to copy into.
    name : str
        Name of the new object. If omitted, name of the original will be used.
    n : int
        Number of copies to make (default=1).
    toGlobal : bool
        Relevant for parallel environments only. If false, the copies will
        reside on local node, otherwise all nodes get the copies.
    copyExtMsg : bool
        If true, messages to/from external objects are also copied.

    Returns
    -------
    vec
        newly copied vec
    """
    if isinstance(src, str):
        src = _moose.element(src)
    if isinstance(dest, str):
        dest = _moose.element(dest)
    if not name:
        name = src.name
    return _moose.copy(src.id, dest, name, n, toGlobal, copyExtMsg)


def rand(a=0.0, b=1.0):
    """Generate random number from the interval [0.0, 1.0)

    Returns
    -------
    float in [0, 1) real interval generated by MT19937.

    See also
    --------
    moose.seed() : reseed the random number generator.

    Notes
    -----
    MOOSE does not automatically seed the random number generator. You
    must explicitly call moose.seed() to create a new sequence of random
    numbers each time.
    """
    return _moose.rand(a, b)


def seed(seed=0):
    """Reseed MOOSE random number generator.

    Parameters
    ----------
    seed : int
        Value to use for seeding.
        default: random number generated using system random device

    Notes
    -----
    All RNGs in moose except rand functions in moose.Function expression use
    this seed.

    By default (when this function is not called) seed is initializecd to some
    random value using system random device (if available).

    Returns
    -------
    None

    See also
    --------
    moose.rand() : get a pseudorandom number in the [0,1) interval.
    """
    _moose.seed(seed)


def pwe():
    """Print present working element's path.

    Convenience function for GENESIS users. If you want to retrieve the element
    in stead of printing the path, use moose.getCwe().

    Returns
    ------
    melement
        current MOOSE element

    Example
    -------
    >>> pwe()
    '/'
    """
    pwe_ = _moose.getCwe()
    print(pwe_.path)
    return pwe_


def le(el=None):
    """List elements under `el` or current element if no argument
    specified.

    Parameters
    ----------
    el : str/melement/vec/None

        The element or the path under which to look. If `None`, children of
        current working element are displayed.

    Returns
    -------
    List[str]
        path of all children
    """
    el = _moose.getCwe() if el is None else el
    if isinstance(el, str):
        el = _moose.element(el)
    elif isinstance(el, _moose.vec):
        el = el[0]
    return _moose.le(el)


def showfields(el, field="*", showtype=False):
    """Show the fields of the element `el`, their data types and
    values in human readable format. Convenience function for GENESIS
    users.

    Parameters
    ----------
    el : melement/str
        Element or path of an existing element.

    field : str
        Field to be displayed. If '*' (default), all fields are displayed.

    showtype : bool
        If True show the data type of each field. False by default.

    Returns
    -------
    string

    """
    if isinstance(el, str):
        if not _moose.exists(el):
            raise ValueError("no such element: %s" % el)
        el = _moose.element(el)
    result = []
    if field == "*":
        value_field_dict = _moose.getFieldDict(el.className, "valueFinfo")
        max_type_len = max(len(dtype) for dtype in value_field_dict.values())
        max_field_len = max(len(dtype) for dtype in value_field_dict.keys())
        result.append("\n[" + el.path + "]\n")
        for key, dtype in sorted(value_field_dict.items()):
            if (
                dtype == "bad"
                or key == "this"
                or key == "dummy"
                or key == "me"
                or dtype.startswith("vector")
                or "ObjId" in dtype
            ):
                continue
            value = el.getField(key)
            if showtype:
                typestr = dtype.ljust(max_type_len + 4)
                ## The following hack is for handling both Python 2 and
                ## 3. Directly putting the print command in the if/else
                ## clause causes syntax error in both systems.
                result.append(typestr + " ")
            result.append(key.ljust(max_field_len + 4) + "=" + str(value) + "\n")
    else:
        try:
            result.append(field + "=" + el.getField(field))
        except AttributeError:
            pass  # Genesis silently ignores non existent fields
    print("".join(result))
    return "".join(result)


def showfield(el, field="*", showtype=False):
    """Alias for showfields."""
    return showfields(el, field, showtype)


def listmsg(arg):
    """Return a list containing the incoming and outgoing messages of
    `el`.

    Parameters
    ----------
    arg : melement/vec/str
        MOOSE object or path of the object to look into.

    Returns
    -------
    msg : list
        List of Msg objects corresponding to incoming and outgoing connections
        of `arg`.

    """
    obj = _moose.element(arg)
    assert obj
    return _moose.listmsg(obj)


def showmsg(el):
    """Print the incoming and outgoing messages of `el`.

    Parameters
    ----------
    el : melement/vec/str
        Object whose messages are to be displayed.

    Returns
    -------
    None

    """
    print(_moose.showmsg(_moose.element(el)))


def doc(arg, paged=True):
    """Display the documentation for class or field in a class.

    Parameters
    ----------
    arg : str/class/melement/vec
        A string specifying a moose class name and a field name
        separated by a dot. e.g., 'Neutral.name'. Prepending `moose.`
        is allowed. Thus moose.doc('moose.Neutral.name') is equivalent
        to the above.
        It can also be string specifying just a moose class name or a
        moose class or a moose object (instance of melement or vec
        or there subclasses). In that case, the builtin documentation
        for the corresponding moose class is displayed.

    paged: bool
        Whether to display the docs via builtin pager or print and
        exit. If not specified, it defaults to False and
        moose.doc(xyz) will print help on xyz and return control to
        command line.

    Returns
    -------
    None

    Raises
    ------
    NameError
        If class or field does not exist.

    """
    text = _moose.__generatedoc__(arg)
    if pydoc.page:
        pydoc.pager(text) 
    else:
        print(text)


# SBML related functions.
def readSBML(filepath, loadpath, solver="ee", validate=True):
    """Load SBML model.

    Parameters
    ----------
    filepath: str
        filepath to be loaded.
    loadpath : str
        Root path for this model e.g. /model/mymodel
    solver : str
        Solver to use (default 'ee').
        Available options are "ee", "gsl", "stochastic", "gillespie"
            "rk", "deterministic"
            For full list see ??
    validate: bool
        When True, run the schema validation.
    """
    return model_utils.mooseReadSBML(filepath, loadpath, solver, validate)


def writeSBML(modelpath, filepath, sceneitems={}):
    """Writes loaded model under modelpath to a file in SBML format.

    Parameters
    ----------
    modelpath : str
        model path in moose e.g /model/mymodel
    filepath : str
        Path of output file.
    sceneitems : dict
        UserWarning: user need not worry about this layout position is saved in
        Annotation field of all the moose Object (pool,Reaction,enzyme).
        If this function is called from
        * GUI - the layout position of moose object is passed
        * command line - NA
        * if genesis/kkit model is loaded then layout position is taken from the file
        * otherwise auto-coordinates is used for layout position.
    """
    return model_utils.mooseWriteSBML(modelpath, filepath, sceneitems)


def writeKkit(modelpath, filepath, sceneitems={}):
    """Writes  loded model under modelpath to a file in Kkit format.

    Parameters
    ----------
    modelpath : str
        Model path in moose.
    filepath : str
        Path of output file.
    """
    return model_utils.mooseWriteKkit(modelpath, filepath, sceneitems)


def readNML2(modelpath, verbose=False):
    """Load neuroml2 model.

    Parameters
    ----------
    modelpath: str
        Path of nml2 file.

    verbose: True
        (defalt False)
        If True, enable verbose logging.

    Raises
    ------
    FileNotFoundError: If modelpath is not found or not readable.
    """
    return model_utils.mooseReadNML2(modelpath, verbose)


def writeNML2(outfile):
    """Write model to NML2. (Not implemented)
    """
    raise NotImplementedError("Writing to NML2 is not supported yet")


def addChemSolver(modelpath, solver):
    """Add solver on chemical compartment and its children for calculation.
    (For developers)

    Parameters
    ----------
    modelpath : str
        Model path that is loaded into moose.
    solver : str
        Exponential Euler "ee" is default. Other options are Gillespie ("gssa"),
        Runge Kutta ("gsl"/"rk"/"rungekutta").

    TODO
    ----
    Documentation

    See also
    --------
    deleteChemSolver
    """
    return model_utils.mooseAddChemSolver(modelpath, solver)


def deleteChemSolver(modelpath):
    """Deletes solver on all the compartment and its children

    Notes
    -----
    This is neccesary while created a new moose object on a pre-existing modelpath,
    this should be followed by mooseAddChemSolver for add solvers on to compartment
    to simulate else default is Exponential Euler (ee)

    See also
    --------
    addChemSolver
    """
    return model_utils.mooseDeleteChemSolver(modelpath)


def mergeChemModel(modelpath, dest):
    """Merges two models.

    Merge chemical model in a file `modelpath` with existing MOOSE model at
    path `dest`.

    Parameters
    ----------
    modelpath : str
        Filepath containing a chemical model.
    dest : path
        Existing MOOSE path.

    TODO
    ----
        No example file which shows its use. Deprecated?
    """
    return model_utils.mooseMergeChemModel(modelpath, dest)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

# Bring everything from c++ module to global namespace. 
from moose._moose import *

# Bring everything from moose.py to global namespace. 
# IMP: It will overwrite any c++ function with the same name.  We can override
# some C++ here.
from moose.moose import *
from moose.server import *

# SBML and NML2 support.
from moose.model_utils import *

# create a shorthand for version() call here.
__version__ = version()

# C++ core override
from moose.wrapper import *

# Import moose test.
from moose.moose_test import test



# -* coding: utf-8 -*-
# Utility functions for moose.

from __future__ import print_function, division, absolute_import
from __future__ import print_function, division
from __future__ import absolute_import

import types
import parser
import token
import symbol
import math
from datetime import datetime
from collections import defaultdict
import re

import logging

logger_ = logging.getLogger("moose.utils")

import moose
from moose.moose_constants import *
from moose.print_utils import *

try:
    from moose.network_utils import *
except Exception as e:
    logger_.warn("Netowrk utilities are not loaded due to %s" % e)

# Print and Plot utilities.
try:
    from moose.plot_utils import *
except Exception as e:
    logger_.warn("Plot utilities are not loaded due to '%s'" % e)


def create_table_path(model, graph, element, field):

    field = field[0].upper() + field[1:]

    tablePathSuffix = element.path.partition(model.path)[-1]
    if tablePathSuffix.startswith("/"):
        tablePathSuffix = tablePathSuffix[1:]

    tablePathSuffix = tablePathSuffix.replace("/", "_") + "." + field
    # tablePathSuffix = re.sub(
    #     ".", lambda m: {"[": "_", "]": "_"}.get(m.group(), m.group()), tablePathSuffix
    # )
    #if tablePathSuffix.startswith("_0__"):
    #    tablePathSuffix = tablePathSuffix[4:]

    # tablePath = dataroot + '/' +tablePath
    tablePath = graph.path + "/" + tablePathSuffix
    return tablePath


def create_table(tablePath, element, field, tableType):
    """Create table to record `field` from element `element`

    Tables are created under `dataRoot`, the names are generally
    created by removing `/model` in the beginning of `elementPath`
    and replacing `/` with `_`. If this conflicts with an existing
    table, the id value of the target element (elementPath) is
    appended to the name.
    """
    if moose.exists(tablePath):
        table = moose.element(tablePath)
    else:
        if tableType == "Table2":
            table = moose.Table2(tablePath)
        elif tableType == "Table":
            table = moose.Table(tablePath)
        moose.connect(table, "requestOut", element, "get%s" % (field))
    return table


def readtable(table, filename, separator=None):
    """Reads the file specified by filename to fill the MOOSE table object.

    The file can either have one float on each line, in that case the
    table will be filled with values sequentially.
    Or, the file can have
    index value
    on each line. If the separator between index and value is anything other than
    white space, you can specify it in the separator argument."""

    in_file = open(filename)
    ii = 0
    line_no = 0
    for line in in_file:
        line_no = line_no + 1
        tokens = line.split(separator)
        if not tokens:
            continue
        elif len(tokens) == 1:
            table[ii] = float(tokens[0])
        elif len(tokens) == 2:
            table[int(tokens[0])] = float(tokens[1])
        else:
            print(
                "pymoose.readTable(",
                table,
                ",",
                filename,
                ",",
                separator,
                ") - line#",
                line_no,
                " does not fit.",
            )


def getfields(moose_object):
    """Returns a dictionary of the fields and values in this object."""
    field_names = moose_object.getFieldNames("valueFinfo")
    fields = {}
    for name in field_names:
        fields[name] = moose_object.getField(name)
    return fields


def findAllBut(moose_wildcard, stringToExclude):
    allValidObjects = moose.wildcardFind(moose_wildcard)
    refinedList = []
    for validObject in allValidObjects:
        if validObject.path.find(stringToExclude) == -1:
            refinedList.append(validObject)

    return refinedList


def apply_to_tree(moose_wildcard, python_filter=None, value=None):
    """
    Select objects by a moose/genesis wildcard, apply a python filter on them and apply a value on them.

    moose_wildcard - this follows GENESIS convention.

    {path}/#[{condition}] returns all elements directly under {path} that satisfy condition. For example:

    '/mynetwork/mycell_0/#[TYPE=Compartment]'

    will return all Compartment objects directly under mycell_0 in mynetwork.

    '{path}/##[{condition}]' will recursively go through all the
    objects that are under {path} (i.e. children, grandchildren,
    great-grandchildren and so on up to the leaf level) and a list of
    the ones meet {condition} will be obtained.

    Thus, '/mynetwork/##[TYPE=Compartment]' will return all
    compartments under mynetwork or its children, or children thereof
    and so on.

    python_filter - if a single string, it will be taken as a
    fieldname, and value will be assigned to this field. It can also
    be a lambda function returning True or False which will be applied
    to each id in the id list returned by moose wildcard
    search. Remember, the argument to the lambda will be an Id, so it
    is up to you to wrap it into a moose object of appropriate type. An example is:

    lambda moose_id: Compartment(moose_id).diameter <  2e-6

    If your moose_wildcard selected objects of Compartment class, then
    this lambda function will select only those with diameter less
    than 2 um.

    value - can be a lambda function to apply arbitrary operations on
    the selected objects.

    If python_filter is a string it, the return
    value of applying the lambda for value() will assigned to the
    field specified by python_filter.

    But if it is value is a data object and {python_filter} is a
    string, then {value} will be assigned to the field named
    {python_filter}.


    If you want to assign Rm = 1e6 for each compartment in mycell
    whose name match 'axon_*':

    apply_to_tree('/mycell/##[Class=Compartment]',
            lambda x: 'axon_' in Neutral(x).name,
            lambda x: setattr(Compartment(x), 'Rm', 1e6))

    [you must use setattr to assign value to a field because lambda
    functions don't allow assignments].
    """
    if not isinstance(moose_wildcard, str):
        raise TypeError("moose_wildcard must be a string.")
    id_list = moose.getWildcardList(moose_wildcard, True)
    if isinstance(python_filter, types.LambdaType):
        id_list = [moose_id for moose_id in id_list if python_filter(moose_id)]
    elif isinstance(python_filter, str):
        id_list = [
            moose_id
            for moose_id in id_list
            if hasattr(
                eval("moose.%s(moose_id)" % (moose.Neutral(moose_id).className)),
                python_filter,
            )
        ]
    else:
        pass
    if isinstance(value, types.LambdaType):
        if isinstance(python_filter, str):
            for moose_id in id_list:
                moose_obj = eval(
                    "moose.%s(moose_id)" % (moose.Neutral(moose_id).className)
                )
                setattr(moose_obj, python_filter, value(moose_id))
        else:
            for moose_id in id_list:
                value(moose_id)
    else:
        if isinstance(python_filter, str):
            for moose_id in id_list:
                moose_obj = eval(
                    "moose.%s(moose_id)" % (moose.Neutral(moose_id).className)
                )
                setattr(moose_obj, python_filter, value)
        else:
            raise TypeError(
                "Second argument must be a string specifying a field to assign to when third argument is a value"
            )


def tweak_field(moose_wildcard, field, assignment_string):
    """Tweak a specified field of all objects that match the
    moose_wildcard using assignment string. All identifiers in
    assignment string must be fields of the target object.

    Example:

    tweak_field('/mycell/##[Class=Compartment]', 'Rm', '1.5 / (3.1416 * diameter * length')

    will assign Rm to every compartment in mycell such that the
    specific membrane resistance is 1.5 Ohm-m2.
    """
    if not isinstance(moose_wildcard, str):
        raise TypeError("moose_wildcard must be a string.")
    id_list = moose.getWildcardList(moose_wildcard, True)
    expression = parser.expr(assignment_string)
    expr_list = expression.tolist()
    # This is a hack: I just tried out some possible syntax trees and
    # hand coded the replacement such that any identifier is replaced
    # by moose_obj.identifier
    def replace_fields_with_value(x):
        if len(x) > 1:
            if (
                x[0] == symbol.power
                and x[1][0] == symbol.atom
                and x[1][1][0] == token.NAME
            ):
                field = x[1][1][1]
                x[1] = [symbol.atom, [token.NAME, "moose_obj"]]
                x.append([symbol.trailer, [token.DOT, "."], [token.NAME, field]])
            for item in x:
                if isinstance(item, list):
                    replace_fields_with_value(item)
        return x

    tmp = replace_fields_with_value(expr_list)
    new_expr = parser.sequence2st(tmp)
    code = new_expr.compile()
    for moose_id in id_list:
        moose_obj = eval("moose.%s(moose_id)" % (moose.Neutral(moose_id).className))
        value = eval(code)
        moose.setField(moose_id, field, str(value))


# 2012-01-11 19:20:39 (+0530) Subha: checked for compatibility with dh_branch
def printtree(root, vchar="|", hchar="__", vcount=1, depth=0, prefix="", is_last=False):
    """Pretty-print a MOOSE tree.

    root - the root element of the MOOSE tree, must be some derivatine of Neutral.

    vchar - the character printed to indicate vertical continuation of
    a parent child relationship.

    hchar - the character printed just before the node name

    vcount - determines how many lines will be printed between two
    successive nodes.

    depth - for internal use - should not be explicitly passed.

    prefix - for internal use - should not be explicitly passed.

    is_last - for internal use - should not be explicitly passed.

    """
    root = moose.element(root)
    # print('%s: "%s"' % (root, root.children))
    for i in range(vcount):
        print(prefix)

    if depth != 0:
        print(prefix + hchar, end=" ")
        if is_last:
            index = prefix.rfind(vchar)
            prefix = prefix[:index] + " " * (len(hchar) + len(vchar)) + vchar
        else:
            prefix = prefix + " " * len(hchar) + vchar
    else:
        prefix = prefix + vchar

    print(root.name)
    children = []
    for child_vec in root.children:
        try:
            child = moose.element(child_vec)
            children.append(child)
        except TypeError:
            pass
            # print 'TypeError:', child_vec, 'when converting to element.'
    for i in range(0, len(children) - 1):
        printtree(children[i], vchar, hchar, vcount, depth + 1, prefix, False)
    if len(children) > 0:
        printtree(children[-1], vchar, hchar, vcount, depth + 1, prefix, True)


def df_traverse(root, operation, *args):
    """Traverse the tree in a depth-first manner and apply the
    operation using *args. The first argument is the root object by
    default."""
    if hasattr(root, "_visited"):
        return
    operation(root, *args)
    for child in root.children:
        childNode = moose.Neutral(child)
        df_traverse(childNode, operation, *args)
    root._visited = True


def autoposition(root):
    """Automatically set the positions of the endpoints of all the
    compartments under `root`.

    This keeps x and y constant and extends the positions in
    z-direction only. This of course puts everything in a single line
    but suffices for keeping electrical properties intact.

    TODO: in future we may want to create the positions for nice
    visual layout as well. My last attempt resulted in some
    compartments overlapping in space.

    """
    compartments = moose.wildcardFind("%s/##[TYPE=Compartment]" % (root.path))
    stack = [
        compartment
        for compartment in map(moose.element, compartments)
        if len(compartment.neighbors["axial"]) == 0
    ]

    assert len(stack) == 1, (
        "There must be one and only one top level\
            compartment. Found %d"
        % len(stack)
    )
    ret = stack[0]
    while len(stack) > 0:
        comp = stack.pop()
        parentlist = comp.neighbors["axial"]
        parent = None
        if len(parentlist) > 0:
            parent = parentlist[0]
            comp.x0, comp.y0, comp.z0, = parent.x, parent.y, parent.z
        else:
            comp.x0, comp.y0, comp.z0, = (0, 0, 0)
        if comp.length > 0:
            comp.x, comp.y, comp.z, = comp.x0, comp.y0, comp.z0 + comp.length
        else:
            # for spherical compartments x0, y0, z0 are centre
            # position nad x,y,z are on the surface
            comp.x, comp.y, comp.z, = comp.x0, comp.y0, comp.z0 + comp.diameter / 2.0
        # We take z == 0 as an indicator that this compartment has not
        # been processed before - saves against inadvertent loops.
        stack.extend(
            [
                childcomp
                for childcomp in map(moose.element, comp.neighbors["raxial"])
                if childcomp.z == 0
            ]
        )
    return ret


def loadModel(filename, target, method="ee"):
    moose.loadModel(filename, target)
    moose.mooseAddChemSolver(target, method)
    if moose.exists(target + "/kinetics/info"):
        moose.element(target + "/kinetics/info").solver = method


def readcell_scrambled(filename, target, method="ee"):
    """A special version for handling cases where a .p file has a line
    with specified parent yet to be defined.

    It creates a temporary file with a sorted version based on
    connectivity, so that parent is always defined before child."""
    pfile = open(filename, "r")
    tmpfilename = filename + ".tmp"
    graph = defaultdict(list)
    data = {}
    error = None
    root = None
    ccomment_started = False
    current_compt_params = []
    for line in pfile:
        tmpline = line.strip()
        if not tmpline or tmpline.startswith("//"):
            continue
        elif tmpline.startswith("/*"):
            ccomment_started = True
        if tmpline.endswith("*/"):
            ccomment_started = False
        if ccomment_started:
            continue
        if tmpline.startswith("*set_compt_param"):
            current_compt_params.append(tmpline)
            continue
        node, parent, rest, = tmpline.partition(" ")
        print("22222222", node, parent)
        if parent == "none":
            if root is None:
                root = node
            else:
                raise ValueError(
                    "Duplicate root elements: ",
                    root,
                    node,
                    "> Cannot process any further.",
                )
                break
        graph[parent].append(node)
        data[node] = "\n".join(current_compt_params)

    tmpfile = open(tmpfilename, "w")
    stack = [root]
    while stack:
        current = stack.pop()
        children = graph[current]
        stack.extend(children)
        print('#########"', current, '": ', data[current])
        tmpfile.write(data[current])
    tmpfile.close()
    ret = moose.loadModel(tmpfilename, target, method)
    return ret


## Subha: In many scenarios resetSim is too rigid and focussed on
## neuronal simulation.  The setDefaultDt and
## assignTicks/assignDefaultTicks keep the process of assigning dt to
## ticks and assigning ticks to objects separate. reinit() should be
## explicitly called by user just before running a simulation, and not
## when setting it up.
def updateTicks(tickDtMap):
    """Try to assign dt values to ticks.

    Parameters
    ----------
    tickDtMap: dict
    map from tick-no. to dt value. if it is empty, then default dt
    values are assigned to the ticks.

    """
    for tickNo, dt in list(tickDtMap.items()):
        if tickNo >= 0 and dt > 0.0:
            moose.setClock(tickNo, dt)
    if all([(v == 0) for v in list(tickDtMap.values())]):
        setDefaultDt()


def assignTicks(tickTargetMap):
    """
    Assign ticks to target elements.

    Parameters
    ----------
    tickTargetMap:
    Map from tick no. to target path and method. The path can be wildcard expression also.
    """
    if len(tickTargetMap) == 0:
        assignDefaultTicks()
    for tickNo, target in list(tickTargetMap.items()):
        if not isinstance(target, str):
            if len(target) == 1:
                moose.useClock(tickNo, target[0], "process")
            elif len(target) == 2:
                moose.useClock(tickNo, target[0], target[1])
        else:
            moose.useClock(tickNo, target, "process")

    # # This is a hack, we need saner way of scheduling
    # ticks = moose.vec('/clock/tick')
    # valid = []
    # for ii in range(ticks[0].localNumField):
    #     if ticks[ii].dt > 0:
    #         valid.append(ii)
    # if len(valid) == 0:
    #     assignDefaultTicks()


def setDefaultDt(elecdt=1e-5, chemdt=0.01, tabdt=1e-5, plotdt1=1.0, plotdt2=0.25e-3):
    """Setup the ticks with dt values.

    Parameters
    ----------

    elecdt: dt for ticks used in computing electrical biophysics, like
    neuronal compartments, ion channels, synapses, etc.

    chemdt: dt for chemical computations like enzymatic reactions.

    tabdt: dt for lookup tables

    plotdt1: dt for chemical kinetics plotting

    plotdt2: dt for electrical simulations

    """
    moose.setClock(0, elecdt)
    moose.setClock(1, elecdt)
    moose.setClock(2, elecdt)
    moose.setClock(3, elecdt)
    moose.setClock(4, chemdt)
    moose.setClock(5, chemdt)
    moose.setClock(6, tabdt)
    moose.setClock(7, tabdt)
    moose.setClock(8, plotdt1)  # kinetics sim
    moose.setClock(9, plotdt2)  # electrical sim


def assignDefaultTicks(modelRoot="/model", dataRoot="/data", solver="hsolve"):
    if not isinstance(modelRoot, str):
        modelRoot = modelRoot.path
    if not isinstance(dataRoot, str):
        dataRoot = dataRoot.path
    if (
        solver != "hsolve"
        or len(moose.wildcardFind("%s/##[ISA=HSolve]" % (modelRoot))) == 0
    ):
        moose.useClock(0, "%s/##[ISA=Compartment]" % (modelRoot), "init")
        moose.useClock(1, "%s/##[ISA=Compartment]" % (modelRoot), "process")
        moose.useClock(2, "%s/##[ISA=HHChannel]" % (modelRoot), "process")
        # moose.useClock(2, '%s/##[ISA=ChanBase]'  % (modelRoot), 'process')
    moose.useClock(0, "%s/##[ISA=IzhikevichNrn]" % (modelRoot), "process")
    moose.useClock(0, "%s/##[ISA=GapJunction]" % (modelRoot), "process")
    moose.useClock(0, "%s/##[ISA=HSolve]" % (modelRoot), "process")
    moose.useClock(1, "%s/##[ISA=LeakyIaF]" % (modelRoot), "process")
    moose.useClock(1, "%s/##[ISA=IntFire]" % (modelRoot), "process")
    moose.useClock(1, "%s/##[ISA=SpikeGen]" % (modelRoot), "process")
    moose.useClock(1, "%s/##[ISA=PulseGen]" % (modelRoot), "process")
    moose.useClock(1, "%s/##[ISA=StimulusTable]" % (modelRoot), "process")
    moose.useClock(1, "%s/##[ISA=TimeTable]" % (modelRoot), "process")
    moose.useClock(2, "%s/##[ISA=HHChannel2D]" % (modelRoot), "process")
    moose.useClock(2, "%s/##[ISA=SynChan]" % (modelRoot), "process")
    moose.useClock(2, "%s/##[ISA=MgBlock]" % (modelRoot), "process")
    moose.useClock(3, "%s/##[ISA=CaConc]" % (modelRoot), "process")
    moose.useClock(3, "%s/##[ISA=Func]" % (modelRoot), "process")
    # The voltage clamp circuit depends critically on the dt used for
    # computing soma Vm and need to be on a clock with dt=elecdt.
    moose.useClock(0, "%s/##[ISA=DiffAmp]" % (modelRoot), "process")
    moose.useClock(0, "%s/##[ISA=VClamp]" % (modelRoot), "process")
    moose.useClock(0, "%s/##[ISA=PIDController]" % (modelRoot), "process")
    moose.useClock(0, "%s/##[ISA=RC]" % (modelRoot), "process")
    # Special case for kinetics models
    kinetics = moose.wildcardFind("%s/##[FIELD(name)=kinetics]" % modelRoot)
    if len(kinetics) > 0:
        # Do nothing for kinetics models - until multiple scheduling issue is fixed.
        moose.useClock(4, "%s/##[ISA!=PoolBase]" % (kinetics[0].path), "process")
        moose.useClock(5, "%s/##[ISA==PoolBase]" % (kinetics[0].path), "process")
        moose.useClock(18, "%s/##[ISA=Table2]" % (dataRoot), "process")
    else:
        # input() function is called in Table. process() which gets
        # called at each timestep. When a message is connected
        # explicitly to input() dest field, it is driven by the sender
        # and process() adds garbage value to the vector. Hence not to
        # be scheduled.
        for tab in moose.wildcardFind("%s/##[ISA=Table]" % (dataRoot)):
            if len(tab.neighbors["input"]) == 0:
                moose.useClock(9, tab.path, "process")


def stepRun(simtime, steptime, verbose=True, logger=None):
    """Run the simulation in steps of `steptime` for `simtime`."""
    global logger_
    if logger is None:
        logger = logger_
    clock = moose.element("/clock")
    if verbose:
        msg = "Starting simulation for %g" % (simtime)
        logger_.info(msg)
    ts = datetime.now()
    while clock.currentTime < simtime - steptime:
        ts1 = datetime.now()
        moose.start(steptime)
        te = datetime.now()
        td = te - ts1
        if verbose:
            msg = "Simulated till %g. Left: %g. %g of simulation took: %g s" % (
                clock.currentTime,
                simtime - clock.currentTime,
                steptime,
                td.days * 86400 + td.seconds + 1e-6 * td.microseconds,
            )
            logger_.info(msg)

    remaining = simtime - clock.currentTime
    if remaining > 0:
        if verbose:
            msg = "Running the remaining %g." % (remaining)
            logger_.info(msg)
        moose.start(remaining)
    te = datetime.now()
    td = te - ts
    dt = min([t for t in moose.element("/clock").dts if t > 0.0])
    if verbose:
        msg = "Finished simulation of %g with minimum dt=%g in %g s" % (
            simtime,
            dt,
            td.days * 86400 + td.seconds + 1e-6 * td.microseconds,
        )
        logger_.info(msg)


############# added by Aditya Gilra -- begin ################


def resetSim(simpaths, simdt, plotdt, simmethod="hsolve"):
    """ For each of the MOOSE paths in simpaths, this sets the clocks and finally resets MOOSE.
    If simmethod=='hsolve', it sets up hsolve-s for each Neuron under simpaths, and clocks for hsolve-s too. """
    print("Solver:", simmethod)
    moose.setClock(INITCLOCK, simdt)
    moose.setClock(ELECCLOCK, simdt)  # The hsolve and ee methods use clock 1
    moose.setClock(
        CHANCLOCK, simdt
    )  # hsolve uses clock 2 for mg_block, nmdachan and others.
    moose.setClock(POOLCLOCK, simdt)  # Ca/ion pools & funcs use clock 3
    moose.setClock(STIMCLOCK, simdt)  # Ca/ion pools & funcs use clock 3
    moose.setClock(PLOTCLOCK, plotdt)  # for tables
    for simpath in simpaths:
        ## User can connect [qty]Out of an element to input of Table or
        ## requestOut of Table to get[qty] of the element.
        ## Scheduling the Table to a clock tick, will call process() of the Table
        ## which will send a requestOut and overwrite any value set by input(),
        ## thus adding garbage value to the vector. Hence schedule only if
        ## input message is not connected to the Table.
        for table in moose.wildcardFind(simpath + "/##[TYPE=Table]"):
            if len(table.neighbors["input"]) == 0:
                moose.useClock(PLOTCLOCK, table.path, "process")
        moose.useClock(ELECCLOCK, simpath + "/##[TYPE=PulseGen]", "process")
        moose.useClock(STIMCLOCK, simpath + "/##[TYPE=DiffAmp]", "process")
        moose.useClock(STIMCLOCK, simpath + "/##[TYPE=VClamp]", "process")
        moose.useClock(STIMCLOCK, simpath + "/##[TYPE=PIDController]", "process")
        moose.useClock(STIMCLOCK, simpath + "/##[TYPE=RC]", "process")
        moose.useClock(STIMCLOCK, simpath + "/##[TYPE=TimeTable]", "process")
        moose.useClock(ELECCLOCK, simpath + "/##[TYPE=LeakyIaF]", "process")
        moose.useClock(ELECCLOCK, simpath + "/##[TYPE=IntFire]", "process")
        moose.useClock(ELECCLOCK, simpath + "/##[TYPE=IzhikevichNrn]", "process")
        moose.useClock(ELECCLOCK, simpath + "/##[TYPE=SpikeGen]", "process")
        moose.useClock(ELECCLOCK, simpath + "/##[TYPE=Interpol]", "process")
        moose.useClock(ELECCLOCK, simpath + "/##[TYPE=Interpol2D]", "process")
        moose.useClock(CHANCLOCK, simpath + "/##[TYPE=HHChannel2D]", "process")
        moose.useClock(CHANCLOCK, simpath + "/##[TYPE=SynChan]", "process")
        ## If simmethod is not hsolve, set clocks for the biophysics,
        ## else just put a clock on the hsolve:
        ## hsolve takes care of the clocks for the biophysics
        if "hsolve" not in simmethod.lower():
            print("Using exp euler")
            moose.useClock(INITCLOCK, simpath + "/##[TYPE=Compartment]", "init")
            moose.useClock(ELECCLOCK, simpath + "/##[TYPE=Compartment]", "process")
            moose.useClock(CHANCLOCK, simpath + "/##[TYPE=HHChannel]", "process")
            moose.useClock(POOLCLOCK, simpath + "/##[TYPE=CaConc]", "process")
            moose.useClock(POOLCLOCK, simpath + "/##[TYPE=Func]", "process")
        else:  # use hsolve, one hsolve for each Neuron
            print("Using hsolve")
            element = moose.Neutral(simpath)
            for childid in element.children:
                childobj = moose.element(childid)
                classname = childobj.className
                if classname in ["Neuron"]:
                    neuronpath = childobj.path
                    h = moose.HSolve(neuronpath + "/solve")
                    h.dt = simdt
                    h.target = neuronpath
                    moose.useClock(INITCLOCK, h.path, "process")
    moose.reinit()


def setupTable(name, obj, qtyname, tables_path=None, threshold=None, spikegen=None):
    """ Sets up a table with 'name' which stores 'qtyname' field from 'obj'.
    The table is created under tables_path if not None, else under obj.path . """
    if tables_path is None:
        tables_path = obj.path + "/data"
    ## in case tables_path does not exist, below wrapper will create it
    tables_path_obj = moose.Neutral(tables_path)
    qtyTable = moose.Table(tables_path_obj.path + "/" + name)
    assert qtyTable, "%s not found" % qtyTable
    ## stepMode no longer supported, connect to 'input'/'spike' message dest to record Vm/spiktimes
    # qtyTable.stepMode = TAB_BUF
    if spikegen is None:
        if threshold is None:
            ## below is wrong! reads qty twice every clock tick!
            # moose.connect( obj, qtyname+'Out', qtyTable, "input")
            ## this is the correct method
            moose.connect(qtyTable, "requestOut", obj, "get" + qtyname)
        else:
            ## create new spikegen
            spikegen = moose.SpikeGen(tables_path_obj.path + "/" + name + "_spikegen")
            ## connect the compartment Vm to the spikegen
            moose.connect(obj, "VmOut", spikegen, "Vm")
            ## spikegens for different synapse_types can have different thresholds
            spikegen.threshold = threshold
            spikegen.edgeTriggered = (
                1  # This ensures that spike is generated only on leading edge.
            )
    else:
        moose.connect(
            spikegen, "spikeOut", qtyTable, "input"
        )  ## spikeGen gives spiketimes
    return qtyTable


def connectSynapse(compartment, synname, gbar_factor):
    """
    Creates a synname synapse under compartment, sets Gbar*gbar_factor, and attaches to compartment.
    synname must be a synapse in /library of MOOSE.
    """
    synapseid = moose.copy(moose.SynChan("/library/" + synname), compartment, synname)
    synapse = moose.SynChan(synapseid)
    synapse.Gbar = synapse.Gbar * gbar_factor
    synapse_mgblock = moose.Mstring(synapse.path + "/mgblockStr")
    if (
        synapse_mgblock.value == "True"
    ):  # If NMDA synapse based on mgblock, connect to mgblock
        mgblock = moose.Mg_block(synapse.path + "/mgblock")
        compartment.connect("channel", mgblock, "channel")
    else:
        compartment.connect("channel", synapse, "channel")
    return synapse


def printNetTree():
    """ Prints all the cells under /, and recursive prints the cell tree for each cell. """
    root = moose.Neutral("/")
    for id in root.children:  # all subelements of 'root'
        if moose.Neutral(id).className == "Cell":
            cell = moose.Cell(id)
            print(
                "-------------------- CELL : ",
                cell.name,
                " ---------------------------",
            )
            printCellTree(cell)


def printCellTree(cell):
    """
    Prints the tree under MOOSE object 'cell'.
    Assumes cells have all their compartments one level below,
    also there should be nothing other than compartments on level below.
    Apart from compartment properties and messages,
    it displays the same for subelements of compartments only one level below the compartments.
    Thus NMDA synapses' mgblock-s will be left out.

    FIXME: no lenght cound on compartment.
    """
    for compartmentid in cell.children:  # compartments
        comp = moose.Compartment(compartmentid)
        print(
            "  |-",
            comp.path,
            "l=",
            comp.length,
            "d=",
            comp.diameter,
            "Rm=",
            comp.Rm,
            "Ra=",
            comp.Ra,
            "Cm=",
            comp.Cm,
            "EM=",
            comp.Em,
        )
        # for inmsg in comp.inMessages():
        #    print "    |---", inmsg
        # for outmsg in comp.outMessages():
        #    print "    |---", outmsg
        printRecursiveTree(
            compartmentid, level=2
        )  # for channels and synapses and recursively lower levels


## Use printCellTree which calls this
def printRecursiveTree(elementid, level):
    """ Recursive helper function for printCellTree,
    specify depth/'level' to recurse and print subelements under MOOSE 'elementid'. """
    spacefill = "  " * level
    element = moose.Neutral(elementid)
    for childid in element.children:
        childobj = moose.Neutral(childid)
        classname = childobj.className
        if classname in ["SynChan", "KinSynChan"]:
            childobj = moose.SynChan(childid)
            print(
                spacefill + "|--",
                childobj.name,
                childobj.className,
                "Gbar=",
                childobj.Gbar,
                "numSynapses=",
                childobj.numSynapses,
            )
            return  # Have yet to figure out the children of SynChan, currently not going deeper
        elif classname in ["HHChannel", "HHChannel2D"]:
            childobj = moose.HHChannel(childid)
            print(
                spacefill + "|--",
                childobj.name,
                childobj.className,
                "Gbar=",
                childobj.Gbar,
                "Ek=",
                childobj.Ek,
            )
        elif classname in ["CaConc"]:
            childobj = moose.CaConc(childid)
            print(
                spacefill + "|--",
                childobj.name,
                childobj.className,
                "thick=",
                childobj.thick,
                "B=",
                childobj.B,
            )
        elif classname in ["Mg_block"]:
            childobj = moose.Mg_block(childid)
            print(
                spacefill + "|--",
                childobj.name,
                childobj.className,
                "CMg",
                childobj.CMg,
                "KMg_A",
                childobj.KMg_A,
                "KMg_B",
                childobj.KMg_B,
            )
        elif classname in ["SpikeGen"]:
            childobj = moose.SpikeGen(childid)
            print(
                spacefill + "|--",
                childobj.name,
                childobj.className,
                "threshold",
                childobj.threshold,
            )
        elif classname in ["Func"]:
            childobj = moose.Func(childid)
            print(
                spacefill + "|--",
                childobj.name,
                childobj.className,
                "expr",
                childobj.expr,
            )
        elif classname in [
            "Table"
        ]:  # Table gives segfault if printRecursiveTree is called on it
            return  # so go no deeper
        # for inmsg in childobj.inMessages():
        #    print spacefill+"  |---", inmsg
        # for outmsg in childobj.outMessages():
        #    print spacefill+"  |---", outmsg
        if len(childobj.children) > 0:
            printRecursiveTree(childid, level + 1)


def setup_vclamp(compartment, name, delay1, width1, level1, gain=0.5e-5):
    """
    Sets up a voltage clamp with 'name' on MOOSE 'compartment' object:
    adapted from squid.g in DEMOS (moose/genesis)
    Specify the 'delay1', 'width1' and 'level1' of the voltage to be applied to the compartment.
    Typically you need to adjust the PID 'gain'
    For perhaps the Davison 4-compartment mitral or the Davison granule:
    0.5e-5 optimal gain - too high 0.5e-4 drives it to oscillate at high frequency,
    too low 0.5e-6 makes it have an initial overshoot (due to Na channels?)
    Returns a MOOSE table with the PID output.
    """
    ## If /elec doesn't exists it creates /elec and returns a reference to it.
    ## If it does, it just returns its reference.
    moose.Neutral("/elec")
    pulsegen = moose.PulseGen("/elec/pulsegen" + name)
    vclamp = moose.DiffAmp("/elec/vclamp" + name)
    vclamp.saturation = 999.0
    vclamp.gain = 1.0
    lowpass = moose.RC("/elec/lowpass" + name)
    lowpass.R = 1.0
    lowpass.C = 50e-6  # 50 microseconds tau
    PID = moose.PIDController("/elec/PID" + name)
    PID.gain = gain
    PID.tau_i = 20e-6
    PID.tau_d = 5e-6
    PID.saturation = 999.0
    # All connections should be written as source.connect('',destination,'')
    pulsegen.connect("outputSrc", lowpass, "injectMsg")
    lowpass.connect("outputSrc", vclamp, "plusDest")
    vclamp.connect("outputSrc", PID, "commandDest")
    PID.connect("outputSrc", compartment, "injectMsg")
    compartment.connect("VmSrc", PID, "sensedDest")

    pulsegen.trigMode = 0  # free run
    pulsegen.baseLevel = -70e-3
    pulsegen.firstDelay = delay1
    pulsegen.firstWidth = width1
    pulsegen.firstLevel = level1
    pulsegen.secondDelay = 1e6
    pulsegen.secondLevel = -70e-3
    pulsegen.secondWidth = 0.0

    vclamp_I = moose.Table("/elec/vClampITable" + name)
    vclamp_I.stepMode = TAB_BUF  # TAB_BUF: table acts as a buffer.
    vclamp_I.connect("inputRequest", PID, "output")
    vclamp_I.useClock(PLOTCLOCK)

    return vclamp_I


def setup_iclamp(compartment, name, delay1, width1, level1):
    """
    Sets up a current clamp with 'name' on MOOSE 'compartment' object:
    Specify the 'delay1', 'width1' and 'level1' of the current pulse to be applied to the compartment.
    Returns the MOOSE pulsegen that sends the current pulse.
    """
    ## If /elec doesn't exists it creates /elec and returns a reference to it.
    ## If it does, it just returns its reference.
    moose.Neutral("/elec")
    pulsegen = moose.PulseGen("/elec/pulsegen" + name)
    iclamp = moose.DiffAmp("/elec/iclamp" + name)
    iclamp.saturation = 1e6
    iclamp.gain = 1.0
    pulsegen.trigMode = 0  # free run
    pulsegen.baseLevel = 0.0
    pulsegen.firstDelay = delay1
    pulsegen.firstWidth = width1
    pulsegen.firstLevel = level1
    pulsegen.secondDelay = 1e6  # to avoid repeat
    pulsegen.secondLevel = 0.0
    pulsegen.secondWidth = 0.0
    pulsegen.connect("output", iclamp, "plusIn")
    iclamp.connect("output", compartment, "injectMsg")
    return pulsegen


def get_matching_children(parent, names):
    """ Returns non-recursive children of 'parent' MOOSE object
    with their names containing any of the strings in list 'names'. """
    matchlist = []
    for childID in parent.children:
        child = moose.Neutral(childID)
        for name in names:
            if name in child.name:
                matchlist.append(childID)
    return matchlist


def underscorize(path):
    """ Returns: / replaced by underscores in 'path'.
    But async13 branch has indices in the path like [0],
    so just replacing / by _ is not enough,
    should replace [ and ] also by _ """
    path = path.replace("/", "_")
    # remove [0] from path.
    path = path.replace('[0]', '')
    # now remove [\d+] with -\d-'
    return re.sub(r'\[(\d+)\]', '-\1-', path)


def blockChannels(cell, channel_list):
    """
    Sets gmax to zero for channels of the 'cell' specified in 'channel_list'
    Substring matches in channel_list are allowed
    e.g. 'K' should block all K channels (ensure that you don't use capital K elsewhere in your channel name!)
    """
    for compartmentid in cell.children:  # compartments
        comp = moose.Compartment(compartmentid)
        for childid in comp.children:
            child = moose.Neutral(childid)
            if child.className in ["HHChannel", "HHChannel2D"]:
                chan = moose.HHChannel(childid)
                for channame in channel_list:
                    if channame in chan.name:
                        chan.Gbar = 0.0


def get_child_Mstring(mooseobject, mstring):
    for child in mooseobject.children:
        if child.className == "Mstring" and child.name == mstring:
            child = moose.element(child)
            return child
    return None


def connect_CaConc(compartment_list, temperature=None):
    """ Connect the Ca pools and channels within each of the compartments in compartment_list
     Ca channels should have a child Mstring named 'ion' with value set in MOOSE.
     Ca dependent channels like KCa should have a child Mstring called 'ionDependency' with value set in MOOSE.
     Call this only after instantiating cell so that all channels and pools have been created. """
    for compartment in compartment_list:
        caconc = None
        for child in compartment.children:
            if child.className == "CaConc":
                caconc = moose.element(child)
                break
        if caconc is not None:
            child = get_child_Mstring(caconc, "phi")
            if child is not None:
                caconc.B = float(
                    child.value
                )  # B = phi by definition -- see neuroml 1.8.1 defn
            else:
                ## B has to be set for caconc based on thickness of Ca shell and compartment l and dia,
                ## OR based on the Mstring phi under CaConc path.
                ## I am using a translation from Neuron for mitral cell, hence this method.
                ## In Genesis, gmax / (surfacearea*thick) is set as value of B!
                caconc.B = (
                    1
                    / (2 * FARADAY)
                    / (
                        math.pi
                        * compartment.diameter
                        * compartment.length
                        * caconc.thick
                    )
                )
            for neutralwrap in compartment.children:
                if neutralwrap.className == "HHChannel":
                    channel = moose.HHChannel(child)
                    ## If child Mstring 'ion' is present and is Ca, connect channel current to caconc
                    for childid in channel.children:
                        # in async13, gates which have not been created still 'exist'
                        # i.e. show up as a child, but cannot be wrapped.
                        try:
                            child = moose.element(childid)
                            if child.className == "Mstring":
                                child = moose.Mstring(child)
                                assert child
                                if child.name == "ion":
                                    if child.value in ["Ca", "ca"]:
                                        moose.connect(
                                            channel, "IkOut", caconc, "current"
                                        )
                                        # print 'Connected IkOut of',channel.path,'to current of',caconc.path
                                ## temperature is used only by Nernst part here...
                                if child.name == "nernst_str":
                                    nernst = moose.Nernst(channel.path + "/nernst")
                                    nernst_params = child.value.split(",")
                                    nernst.Cout = float(nernst_params[0])
                                    nernst.valence = int(nernst_params[1])
                                    nernst.Temperature = temperature
                                    moose.connect(nernst, "Eout", channel, "setEk")
                                    moose.connect(caconc, "concOut", nernst, "ci")
                                    # print 'Connected Nernst',nernst.path
                        except TypeError:
                            pass

                if neutralwrap.className == "HHChannel2D":
                    channel = moose.HHChannel2D(child)
                    ## If child Mstring 'ionDependency' is present, connect caconc Ca conc to channel
                    for childid in channel.children:
                        # in async13, gates which have not been created still 'exist'
                        # i.e. show up as a child, but cannot be wrapped.
                        try:
                            child = moose.element(childid)
                            if (
                                child.className == "Mstring"
                                and child.name == "ionDependency"
                            ):
                                child = moose.Mstring(child)
                                if child.value in ["Ca", "ca"]:
                                    moose.connect(caconc, "concOut", channel, "concen")
                                    # print 'Connected concOut of',caconc.path,'to concen of',channel.path
                        except TypeError as e:
                            logger_.warning(e)



# -*- coding: utf-8 -*-
import moose
print(( 'Using moose form %s' % moose.__file__ ))


def main():
    solver = "gssa"
    moose.Neutral('/model')
    moose.CubeMesh('/model/kinetics')
    moose.Pool('/model/kinetics/A')

    #delete if exists
    if ( moose.exists( 'model/kinetics/stoich' ) ):
        moose.delete( '/model/kinetics/stoich' )
        moose.delete( '/model/kinetics/ksolve' )

    #create solver
    compt = moose.element( '/model/kinetics' )
    ksolve = moose.Gsolve( '/model/kinetics/ksolve' )
    stoich = moose.Stoich( '/model/kinetics/stoich' )
    stoich.compartment = compt
    stoich.ksolve = ksolve
    stoich.path = "/model/kinetics/##"
    print(" before reinit")
    moose.reinit()
    print(" After reinit")
    moose.start( 10 )
    print( "Done" )

# Run the 'main' if this script is executed standalone.
if __name__ == '__main__':
	main()




# -*- coding: utf-8 -*-
from __future__ import print_function
try:
    from future_builtins import zip
except ImportError:
    pass
from . import moose as _moose

_tick = 8
_base = '/_utils'
_path = _base + '/y{0}'
_counter = 0
_plots = []

_moose.Neutral( _base )

_defaultFields = {
	_moose.Compartment : 'Vm',
	_moose.ZombieCompartment : 'Vm',

	_moose.HHChannel: 'Gk',
	_moose.ZombieHHChannel: 'Gk',

	_moose.HHChannel2D: 'Gk',

	_moose.SynChan: 'Gk',

	_moose.CaConc: 'Ca',
	_moose.ZombieCaConc: 'Ca',

	_moose.Pool: 'conc',
	_moose.ZombiePool: 'conc',
	_moose.ZPool: 'conc',

	_moose.BufPool: 'conc',
	_moose.ZombieBufPool: 'conc',
	_moose.ZBufPool: 'conc',

	_moose.FuncPool: 'conc',
	_moose.ZombieFuncPool: 'conc',
	_moose.ZFuncPool: 'conc',
}

def _defaultField( obj ):
	return _defaultFields[ type( obj ) ]

def setDt( dt ):
	'''-----------
	Description
	-----------
	Sets time-step for recording values.

	---------
	Arguments
	---------
	dt: Time-step for recording values.

	-------
	Returns
	-------
	Nothing.'''
	_moose.setClock( _tick, dt )

class SetupError( Exception ):
	pass

def _time( npoints = None ):
	import numpy

	if npoints is None:
		try:
			npoints = len( _plots[ 0 ].vec )
		except IndexError:
			raise SetupError(
				'List of time-points cannot be constructed because '
				'no plots have been set up yet.'
			)

	begin   = 0.0
	end     = _moose.Clock( '/clock' ).currentTime

	return numpy.linspace( begin, end, npoints )

class _Plot( _moose.Table ):
	def __init__( self, path, obj, field, label ):
		_moose.Table.__init__( self, path )

		self._table = _moose.Table( path )

		self.obj    = obj
		self.field  = field
		self.label  = label

	@property
	def values( self ):
		return self._table.vec

	@property
	def size( self ):
		return len( self.values )

	@property
	def time( self ):
		return _time( self.size )

	def __iter__( self ):
		return iter( self.values )

def record( obj, field = None, label = None ):
	'''
	'''
	global _counter

	# Checking if object is an iterable like list or a tuple, but not a string.
	if hasattr( obj, '__iter__' ):
		return [ record( o, field, label ) for o in obj ]

	if isinstance( obj, str ):
		obj = _moose.element( obj )

	if field is None:
		field = _defaultField( obj )

	path = _path.format( _counter )
	_counter += 1

	p = _Plot( path, obj, field, label )
	_plots.append( p )

	_moose.connect( p, "requestData", obj, 'get_' + field )
	_moose.useClock( _tick, path, "process" )

	return p

def _label( plot, labelFormat = '{path}.{field}' ):
	# Over-ride label format if label has been given explicitly.
	if plot.label:
		labelFormat = plot.label

	return labelFormat.format(
		path  = plot.obj.path,
		name  = plot.obj.name,
		field = plot.field
	)

def _selectedPlots( selected ):
	if selected is None:
		# Returning a copy of this list, instead of reference. The returned
		# list will be manipulated later.
		return _plots[ : ]
	elif isinstance( selected, _Plot ):
		return [ selected ]
	else:
		return selected

def saveCSV(
		fileName,
		selected = None,
		delimiter = '\t',
		header = True,
		headerCommentCharacter = '#',
		labelFormat = '{path}.{field}',
		timeCol = True,
		timeHeader = 'Time',
		fileMode = 'w' ):
	'''
	'''
	import csv
	plots = _selectedPlots( selected )

	if header:
		header = []

		if timeCol:
			header.append( timeHeader )

		for plot in plots:
			header.append( _label( plot, labelFormat ) )

		header[ 0 ] = headerCommentCharacter + header[ 0 ]

	if timeCol:
		plots.insert( 0, _time() )

	with open( fileName, fileMode ) as fout:
		writer = csv.writer( fout, delimiter = delimiter )

		if header:
			writer.writerow( header )

		writer.writerows( list(zip( *plots )) )

def saveXPLOT(
		fileName,
		selected = None,
		labelFormat = '{path}.{field}',
		fileMode = 'w' ):
	'''
	'''
	plots = _selectedPlots( selected )

	with open( fileName, fileMode ) as fout:
		write = lambda line: fout.write( line + '\n' )

		for ( i, plot ) in enumerate( plots ):
			label = '/plotname ' + _label( plot, labelFormat )

			if i > 0:
				write( '' )
			write( '/newplot' )
			write( label )

			for value in plot:
				write( str( value ) )

def show(
		selected = None,
		combine = True,
		labelFormat = '{path}.{field}',
		xLabel = 'Time (s)',
		yLabel = '{field}' ):
	'''
	'''
	try:
		from matplotlib import pyplot as plt
	except ImportError:
		print("Warning: recording.show(): Cannot find 'matplotlib'. Not showing plots.")
		return

	plots = _selectedPlots( selected )

	if combine:
		plt.figure()

	for plot in plots:
		if not combine:
			plt.figure()

		print(_label(plot))
		plt.plot( plot.time, plot.values, label = _label( plot ) )

	plt.legend()
	plt.show()

def HDF5():
	pass



import matplotlib as mpl
mpl.use('Agg')
import numpy as np
import moose
import rdesigneur as rd

def test_current_pulse_squid():
    """Test current pulse.
    >>> test_current_pulse_squid()
    Rdesigneur: Elec model has 1 compartments and 0 spines on 0 compartments.
    [array([-0.065     , -0.06525877, -0.06549723, ..., -0.06676325,
           -0.06676329, -0.06676332])]
    """
    rdes = rd.rdesigneur(
        chanProto = [['make_HH_Na()', 'Na'], ['make_HH_K()', 'K']],
        chanDistrib = [
            ['Na', 'soma', 'Gbar', '1200' ],
            ['K', 'soma', 'Gbar', '360' ]],
        stimList = [['soma', '1', '.', 'inject', '(t>0.1 && t<0.2) * 1e-8' ]],
        plotList = [['soma', '1', '.', 'Vm', 'Membrane potential']])
    rdes.buildModel()
    moose.reinit()
    moose.start(0.3)
    rdes.display(block=False)
    data =  moose.wildcardFind('/##[TYPE=Table]')[0].vector
    m, u = data.mean(), data.std()
    np.allclose([-0.06507780556166297, 0.012544993918549656], [m, u])
    print("[INFO ] Done")
    return True

if __name__ == '__main__':
    test_current_pulse_squid()



# -*- coding: utf-8 -*-
# Test scripts are always called from BUILD directory by cmake.

import os
import sys
import numpy as np
import time
import test_difshells as td
import chan_proto
import param_chan

import moose
print('Using moose from %s' % moose.__file__ )

difshell_no = 3
difbuf_no = 0

script_dir_ = os.path.dirname( os.path.abspath( __file__ ) )

p_file = os.path.join( script_dir_, "soma.p" )

cond = {'CaL12':30*0.35e-5, 'SK':0.5*0.35e-6}

def assert_stat( vec, expected ):
    min_, max_ = np.min( vec ), np.max( vec )
    mean, std = np.mean( vec ), np.std( vec )
    computed = [ min_, max_, mean, std ]
    assert np.allclose( computed, expected ),  \
        "Got %s expected %s" % (computed, expected)

def test_hsolve_calcium():
    for tick in range(0, 7):
        moose.setClock(tick,10e-6)
    moose.setClock(8, 0.005)

    lib = moose.Neutral('/library')
    model = moose.loadModel(p_file,'neuron')
    pulse = moose.PulseGen('/neuron/pulse')
    inject = 100e-10
    chan_proto.chan_proto('/library/SK',param_chan.SK)
    chan_proto.chan_proto('/library/CaL12',param_chan.Cal)
    pulse.delay[0] = 8.
    pulse.width[0] = 500e-12
    pulse.level[0] = inject
    pulse.delay[1] = 1e9

    for comp in moose.wildcardFind('/neuron/#[TYPE=Compartment]'):
        new_comp = moose.element(comp)
        new_comp.initVm = -.08
        difs, difb = td.add_difshells_and_buffers(new_comp,difshell_no,difbuf_no)
        for name in cond:
            chan = td.addOneChan(name,cond[name],new_comp)
            if 'Ca' in name:
                moose.connect(chan, "IkOut", difs[0], "influx")

            if 'SK' in name:
                moose.connect(difs[0], 'concentrationOut', chan, 'concen')

    data = moose.Neutral('/data')
    vmtab = moose.Table('/data/Vm')
    shelltab = moose.Table('/data/Ca')
    caltab = moose.Table('/data/CaL_Gk')
    sktab = moose.Table('/data/SK_Gk')
    moose.connect(vmtab, 'requestOut',moose.element('/neuron/soma') , 'getVm')
    moose.connect(shelltab, 'requestOut', difs[0], 'getC')
    moose.connect(caltab,'requestOut',moose.element('/neuron/soma/CaL12') ,'getGk')
    moose.connect(sktab,'requestOut', moose.element('/neuron/soma/SK'),'getGk')

    hsolve = moose.HSolve('/neuron/hsolve')
    hsolve.dt = 10e-6
    hsolve.target = ('/neuron/soma')
    t_stop = 10.
    moose.reinit()
    moose.start(t_stop)
    vec1 = sktab.vector
    vec2 = shelltab.vector
    assert_stat( vec1, [ 0.0, 5.102834e-22, 4.79066e-22, 2.08408e-23 ] )
    assert_stat( vec2, [ 5.0e-5, 5.075007e-5, 5.036985e-5, 2.1950117e-7] )
    assert len(np.where(sktab.vector<1e-19)[0]) == 2001
    assert len(np.where(shelltab.vector>50e-6)[0]) == 2000

def main():
    test_hsolve_calcium()

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
"""test_random_num.py:

"""

__author__           = "Dilawar Singh"
__copyright__        = "Copyright 2017-, Dilawar Singh"
__version__          = "1.0.0"
__maintainer__       = "Dilawar Singh"
__email__            = "dilawars@ncbs.res.in"
__status__           = "Development"

import sys
import os
import numpy as np
import moose

try:
    reload
except NameError:
    # Python 3
    from importlib import reload

def test_unequal():
    rand1, rand2 = [], []
    N = 10
    for i in range( N ):
        x = moose.rand()
        rand1.append( x )
    reload( moose )
    for i in range( N ):
        x = moose.rand()
        rand2.append( x )

    assert not np.equal( rand1, rand2 ).all()

def test_equal():
    rand1, rand2 = [], []
    N = 10
    moose.seed( 10 )
    for i in range( N ):
        x = moose.rand()
        rand1.append( x )

    reload( moose )
    moose.seed( 10 )
    for i in range( N ):
        x = moose.rand()
        rand2.append( x )

    assert np.equal( rand1, rand2 ).all()

def main( ):
    test_unequal()
    test_equal()

if __name__ == '__main__':
    main()




# -*- coding: utf-8 -*-
""" ChemconnectUtil.py Some of the command function are written """

__author__           = "Harsha Rani"
__copyright__        = "Copyright 2017, Harsha Rani and NCBS Bangalore"
__credits__          = ["NCBS Bangalore"]
__license__          = "GNU GPL"
__version__          = "1.0.0"
__maintainer__       = "Harsha Rani"
__email__            = "hrani@ncbs.res.in"
__status__           = "Development"
__updated__          = "Mar 11 2019"

import moose
import numpy as np
import pickle
import matplotlib
import os
import random
import re

direct = os.path.dirname(__file__)
colormap_file = open(os.path.join(direct, "rainbow2.pkl"), "r")  
colorMap = pickle.loads(colormap_file.read().encode('utf8'))
colormap_file.close()
ignoreColor= ["mistyrose","antiquewhite","aliceblue","azure","bisque","black","blanchedalmond","blue","cornsilk","darkolivegreen","darkslategray","dimgray","floralwhite","gainsboro","ghostwhite","honeydew","ivory","lavender","lavenderblush","lemonchiffon","lightcyan","lightgoldenrodyellow","lightgray","lightyellow","linen","mediumblue","mintcream","navy","oldlace","papayawhip","saddlebrown","seashell","snow","wheat","white","whitesmoke","aquamarine","lightsalmon","moccasin","limegreen","snow","sienna","beige","dimgrey","lightsage"]
matplotcolor = {}
for name,hexno in matplotlib.colors.cnames.items():
    matplotcolor[name]=hexno

def getRandColor():
    k = random.choice(list(matplotcolor.keys()))
    if k in ignoreColor:
        return getRandColor()
    else:
        #return matplotcolor[k]
        return str(matplotlib.colors.cnames[k])

def getColor(iteminfo):
    """ Getting a textcolor and background color for the given  mooseObject \
        If textcolor is empty replaced with green \
           background color is empty replaced with blue
           if textcolor and background is same as it happend in kkit files \
           replacing textcolor with random color\
           The colors are not valid there are siliently replaced with some values \
           but while model building can raise an exception
    """
    textcolor = moose.element(iteminfo).textColor
    bgcolor   = moose.element(iteminfo).color
    if(textcolor == ''): textcolor = 'green'
    
    if(bgcolor == ''): bgcolor = 'orange'

    if(textcolor == bgcolor):
        textcolor = getRandColor()

    textcolor = colorCheck(textcolor)
    bgcolor = colorCheck(bgcolor)
    # if moose.exists(iteminfo):
    #     moose.element(iteminfo).textColor = textcolor
    # moose.element(iteminfo).color = bgcolor
    return(textcolor,bgcolor)

def colorCheck(fc_bgcolor):
    """ textColor or background can be anything like string or tuple or list \
        if string its taken as colorname further down in validColorcheck checked for valid color, \
        but for tuple and list its taken as r,g,b value.
    """
    if isinstance(fc_bgcolor,str):
        if fc_bgcolor.startswith("#"):
            fc_bgcolor = fc_bgcolor
        elif (fc_bgcolor.isdigit()):
            """ color is int  a map from int to r,g,b triplets from pickled color map file """
            tc = (int(fc_bgcolor))*2
            if tc < len(colorMap):
                pickledColor = colorMap[tc]
            else:
                pickledColor = (255, 0, 0)
            fc_bgcolor = '#%02x%02x%02x' % (pickledColor)

        elif fc_bgcolor.isalpha() or fc_bgcolor.isalnum():
            fc_bgcolor = validColorcheck(fc_bgcolor)
            
        else:
            for r in ['[',']','(',')']:
                fc_bgcolor = fc_bgcolor.replace(r,"")
            fc_bgcolor = fc_bgcolor.split(",")
            c = 0
            hexlist ="#"
            for n in fc_bgcolor:
                if c < 3:
                    hexlist = hexlist+str("%02x" % int(n))
                    c = c+1;
            fc_bgcolor = hexlist
        return(fc_bgcolor)

def validColorcheck(color):
    ''' 
        Both in Qt4.7 and 4.8 if not a valid color it makes it as back but in 4.7 there will be a warning mssg which is taken here
        checking if textcolor or backgroundcolor is valid color, if 'No' making white color as default
        where I have not taken care for checking what will be backgroundcolor for textcolor or textcolor for backgroundcolor 
    '''
    #if QColor(color).isValid():
    if matplotlib.colors.is_color_like(color):
        if color == "blue":
            color = "orange"
        color =  matplotlib.colors.cnames[color.lower()]
        return color
    else:
        return(matplotlib.colors.cnames["orange"])

def xyPosition(objInfo,xory):
    if objInfo is None:
        return 0.0
    try:
        return float(moose.element(objInfo).getField(xory))
    except ValueError as e:
        return float(0)

def setupMeshObj(modelRoot):
    ''' Setup compartment and its members pool,reaction,enz cplx under self.meshEntry dictionaries \
    self.meshEntry with "key" as compartment,
    value is key2:list where key2 represents moose object type,list of objects of a perticular type
    e.g self.meshEntry[meshEnt] = { 'reaction': reaction_list,'enzyme':enzyme_list,'pool':poollist,'cplx': cplxlist }
    '''
    xmin = 0.0
    xmax = 1.0
    ymin = 0.0
    ymax = 1.0
    listOfitems = {}
    positionInfoExist = True
    meshEntry = {}
    if meshEntry:
        meshEntry.clear()
    else:
        meshEntry = {}
    xcord = []
    ycord = []
    meshEntryWildcard = '/##[ISA=ChemCompt]'
    if modelRoot != '/':
        meshEntryWildcard = modelRoot+meshEntryWildcard
    for meshEnt in moose.wildcardFind(meshEntryWildcard):
        mollist  = []
        realist  = []
        enzlist  = []
        cplxlist = []
        tablist  = []
        funclist = []

        mol_cpl  = moose.wildcardFind(meshEnt.path+'/##[ISA=PoolBase]')
        funclist = moose.wildcardFind(meshEnt.path+'/##[ISA=Function]')
        enzlist  = moose.wildcardFind(meshEnt.path+'/##[ISA=EnzBase]')
        realist  = moose.wildcardFind(meshEnt.path+'/##[ISA=ReacBase]')
        tablist  = moose.wildcardFind(meshEnt.path+'/##[ISA=StimulusTable]')
        if mol_cpl or funclist or enzlist or realist or tablist:
            for m in mol_cpl:
                objInfo = None
                # if m.parent.isA['CplxEnzBase']: # New bindings
                if moose.element(m.parent).isA['CplxEnzBase']:
                    cplxlist.append(m)
                    objInfo = m.parent.path+'/info'
                elif m.isA['PoolBase']:
                    mollist.append(m)
                    objInfo =m.path+'/info'
                if objInfo is not None and moose.exists(objInfo):
                    listOfitems[moose.element(moose.element(objInfo).parent)]={'x':xyPosition(objInfo,'x'),'y':xyPosition(objInfo,'y')}

                xcord.append(xyPosition(objInfo,'x'))
                ycord.append(xyPosition(objInfo,'y'))

            getxyCord(xcord,ycord,funclist,listOfitems)
            getxyCord(xcord,ycord,enzlist,listOfitems)
            getxyCord(xcord,ycord,realist,listOfitems)
            getxyCord(xcord,ycord,tablist,listOfitems)

            meshEntry[meshEnt] = {'enzyme':enzlist,
                                  'reaction':realist,
                                  'pool':mollist,
                                  'cplx':cplxlist,
                                  'table':tablist,
                                  'function':funclist
                                  }
            positionInfoExist = not(len(np.nonzero(xcord)[0]) == 0 \
                and len(np.nonzero(ycord)[0]) == 0)
            if positionInfoExist:
                xmin = min(xcord)
                xmax = max(xcord)
                ymin = min(ycord)
                ymax = max(ycord)
    return meshEntry,xmin,xmax,ymin,ymax,positionInfoExist,listOfitems
            
def getxyCord(xcord,ycord,list1,listOfitems):
    for item in list1:
        # if isinstance(item,Function):
        #     objInfo = moose.element(item.parent).path+'/info'
        # else:
        #     objInfo = item.path+'/info'
        #if not isinstance(item,moose.Function):
        if not item.isA["Function"]:
            objInfo = item.path+'/info'
            xcord.append(xyPosition(objInfo,'x'))
            ycord.append(xyPosition(objInfo,'y'))
            if moose.exists(objInfo):
                listOfitems[moose.element(moose.element(objInfo).parent)]={'x':xyPosition(objInfo,'x'),'y':xyPosition(objInfo,'y')}

def setupItem(modelPath,cntDict):
    '''This function collects information of what is connected to what. \
    eg. substrate and product connectivity to reaction's and enzyme's \
    sumtotal connectivity to its pool are collected '''
    #print " setupItem"
    sublist = []
    prdlist = []
    zombieType = ['ReacBase','EnzBase','Function','StimulusTable']
    for baseObj in zombieType:
        path = '/##[ISA='+baseObj+']'
        if modelPath != '/':
            path = modelPath+path
        if ( (baseObj == 'ReacBase') or (baseObj == 'EnzBase')):
            for items in moose.wildcardFind(path):
                sublist = []
                prdlist = []
                uniqItem,countuniqItem = countitems(items,'subOut')
                subNo = uniqItem
                for sub in uniqItem:
                    sublist.append((moose.element(sub),'s',countuniqItem[sub]))

                uniqItem,countuniqItem = countitems(items,'prd')
                prdNo = uniqItem
                if (len(subNo) == 0 or len(prdNo) == 0):
                    print ("\nSubstrate Product is empty for "+items.path)

                for prd in uniqItem:
                    prdlist.append((moose.element(prd),'p',countuniqItem[prd]))

                if (baseObj == 'CplxEnzBase') :
                    uniqItem,countuniqItem = countitems(items,'toEnz')
                    for enzpar in uniqItem:
                        sublist.append((moose.element(enzpar),'t',countuniqItem[enzpar]))

                    uniqItem,countuniqItem = countitems(items,'cplxDest')
                    for cplx in uniqItem:
                        prdlist.append((moose.element(cplx),'cplx',countuniqItem[cplx]))

                if (baseObj == 'EnzBase'):
                    uniqItem,countuniqItem = countitems(items,'enzDest')
                    for enzpar in uniqItem:
                        sublist.append((moose.element(enzpar),'t',countuniqItem[enzpar]))
                cntDict[items] = sublist,prdlist
        elif baseObj == 'Function':
            for items in moose.wildcardFind(path):
                sublist = []
                prdlist = []
                item = items.path+'/x[0]'
                uniqItem,countuniqItem = countitems(item,'input')
                for funcpar in uniqItem:
                    sublist.append((moose.element(funcpar),'sts',countuniqItem[funcpar]))

                uniqItem,countuniqItem = countitems(items,'valueOut')
                for funcpar in uniqItem:
                    prdlist.append((moose.element(funcpar),'stp',countuniqItem[funcpar]))
                cntDict[items] = sublist,prdlist

        # elif baseObj == 'Function':
        #     #ZombieSumFunc adding inputs
        #     inputlist = []
        #     outputlist = []
        #     funplist = []
        #     nfunplist = []
        #     for items in moose.wildcardFind(path):
        #         for funplist in moose.element(items).neighbors['valueOut']:
        #             for func in funplist:
        #                 funcx = moose.element(items.path+'/x[0]')
        #                 uniqItem,countuniqItem = countitems(funcx,'input')
        #                 for inPut in uniqItem:
        #                     inputlist.append((inPut,'st',countuniqItem[inPut]))
        #             cntDict[func] = inputlist
        else:
            for tab in moose.wildcardFind(path):
                tablist = []
                uniqItem,countuniqItem = countitems(tab,'output')
                for tabconnect in uniqItem:
                    tablist.append((moose.element(tabconnect),'tab',countuniqItem[tabconnect]))
                cntDict[tab] = tablist

def countitems(mitems,objtype):
    items = []
    items = moose.element(mitems).neighbors[objtype]
    uniqItems = set(items)
    #countuniqItems = Counter(items)
    countuniqItems = dict((i, items.count(i)) for i in items)
    return(uniqItems,countuniqItems)

def findCompartment(elem):
    element = moose.element(elem)
    if element.path == '/':
        return element
    if element.isA['ChemCompt']:
        return element
    return findCompartment(element.parent)

def mooseIsInstance(elem, classes):
    for cl in classes:
        if elem.isA[cl]:
            return True
    return False



# -*- coding: utf-8 -*-
"""
Test MOOSE installation with moose-examples.
"""

from __future__ import print_function, division

__author__    = "Dilawar Singh"
__copyright__ = "Copyright 2016, Dilawar Singh"
__email__     = "dilawars@ncbs.res.in"

import sys
import os
import tempfile
import re
import subprocess
import threading
import signal
import logging
import tempfile
from collections import defaultdict
import time

test_dir_ = os.path.join(tempfile.gettempdir(), 'moose-examples')
test_data_url_ = 'https://github.com/BhallaLab/moose-examples/archive/master.zip'
test_repo_url_ = 'https://github.com/BhallaLab/moose-examples'

ignored_dict_ = defaultdict(list)
test_status_ = defaultdict(list)
total_ = 0

matplotlibrc_ = '''
backend : agg
interactive : True
'''


# Handle CTRL+C
def signal_handler(signal, frame):
    print('You pressed Ctrl+C!')
    print_test_stat()
    quit(-1)


signal.signal(signal.SIGINT, signal_handler)


# Credit: https://stackoverflow.com/a/4825933/1805129
class Command(object):
    def __init__(self, cmd):
        self.cmd = cmd
        self.process = None
        self.fnull = open(os.devnull, 'w')

    def __repr__(self):
        return ' '.join(self.cmd)

    def run(self, timeout, **kwargs):
        self.process = subprocess.Popen(self.cmd, stdout=subprocess.PIPE,
                stderr=subprocess.PIPE)
        timer = threading.Timer(timeout, self.process.kill)
        try:
            timer.start()
            output, errors = self.process.communicate()
            try:
                errors = errors.decode('utf8')
            except Exception:
                pass
            logging.warn('%s %s' % (output, errors))
        finally:
            timer.cancel()
        return self.process.returncode


def init_test_dir():
    global test_dir_
    global test_url_
    if not os.path.exists(test_dir_):
        os.makedirs(test_dir_)
        logging.info("Donwloading test repository")
        subprocess.check_output(
            ['git', 'clone', '--depth=10', test_repo_url_, test_dir_])
    os.chdir(test_dir_)
    subprocess.check_output(['git', 'pull'])


def suitable_for_testing(script):
    with open(script, 'r') as f:
        txt = f.read()
        if not re.search(r'main\(\s*\)', txt):
            logging.debug('Script %s does not contain main( )' % script)
            return False, 'main( ) not found'
        if re.search(r'raw_input\(\s*\)', txt):
            logging.debug('Script %s requires user input' % script)
            return False, 'waits for user input'
    return True, 'OK'


def run_test(index, testfile, timeout, **kwargs):
    """Run a given test
    """
    global test_status_
    global total_
    pyExec = os.environ.get('PYTHON_EXECUTABLE', sys.executable)
    cmd = Command([pyExec, testfile])

    ti = time.time()
    name = os.path.basename(testfile)
    out = (name + '.' * 50)[:40]
    print('[TEST %3d/%d] %41s ' % (index, total_, out), end='')
    sys.stdout.flush()

    # Run the test.
    status = cmd.run(timeout=timeout, **kwargs)
    t = time.time() - ti
    print('% 7.2f s' % t, end='')
    sys.stdout.flush()

    # Change to directory and copy before running then test.
    cwd = os.path.dirname(testfile)
    os.chdir(cwd)
    with open(os.path.join(cwd, 'matplotlibrc'), 'w') as f:
        logging.debug('Writing matplotlibrc to %s' % cwd)
        f.write(matplotlibrc_)

    if status != 0:
        if status == -15:
            msg = '% 4d TIMEOUT' % status
            test_status_['TIMED-OUT'].append(testfile)
        else:
            msg = '% 4d FAILED' % status
            test_status_['FAILED'].append(testfile)
        print(msg)
    else:
        print('% 4d PASSED' % status)
        test_status_['PASSED'].append(testfile)

    sys.stdout.flush()


def print_test_stat():
    global test_status_
    for status in test_status_:
        print('Total %d tests %s' % (len(test_status_[status]), status))


def test_all(timeout, **kwargs):
    global test_dir_
    global total_
    scripts = []
    for d, ds, fs in os.walk(test_dir_):
        for f in fs:
            if not re.match(r'.+\.py$', f):
                continue
            filepath = os.path.join(d, f)
            isOK, msg = suitable_for_testing(filepath)
            if isOK:
                scripts.append(filepath)
            else:
                ignored_dict_[msg].append(filepath)

    for k in ignored_dict_:
        logging.debug('[INFO] Ignored due to %s' % k)
        logging.debug('\n\t'.join(ignored_dict_[k]))

    logging.info('Total %d valid tests found' % len(scripts))
    total_ = len(scripts)
    for i, s in enumerate(scripts):
        logging.info('Running test (timeout=%s) : %s' % (timeout, s))
        run_test(i, s, timeout, **kwargs)


def test(timeout=60, **kwargs):
    """Download and run tests.
    """
    print('[INFO] Running test with timeout %d sec' % timeout)
    try:
        init_test_dir()
    except Exception as e:
        print('[INFO] Failed to clone moose-examples. Error was %s' % e)
        quit()

    test_all(timeout=timeout, **kwargs)
    print_test_stat()


if __name__ == '__main__':
    test()



# -*- coding: utf-8 -*-
# This script can also be called directly to build and install the pymoose
# module.
#
# Alternatively, you can use cmake build system which provides finer control
# over the build. This script is called by cmake to install the python module.
#
# This script is compatible with python2.7 and python3+. Therefore use of
# super() is commented out.
#
# NOTES:
#  * Python2
#   - Update setuptools using `python2 -m pip install setuptools --upgrade --user'.

__author__ = "Dilawar Singh"

__copyright__ = "Copyright 2019-, Dilawar Singh"
__maintainer__ = "Dilawar Singh"
__email__ = "dilawar.s.rajput@gmail.com"

import os
import sys
import multiprocessing
import subprocess
import datetime

try:
    cmakeVersion = subprocess.call(["cmake", "--version"], stdout=subprocess.PIPE)
except Exception as e:
    print(e)
    print("[ERROR] cmake is not found. Please install cmake.")
    quit(-1)

# See https://docs.python.org/3/library/distutils.html
# setuptools is preferred over distutils. And we are supporting python3 only.
from setuptools import setup, Extension, Command
from setuptools.command.build_ext import build_ext as _build_ext
import subprocess

# Global variables.
sdir_ = os.path.dirname(os.path.realpath(__file__))

stamp = datetime.datetime.now().strftime("%Y%m%d")
builddir_ = os.path.join(sdir_, "_temp__build")

if not os.path.exists(builddir_):
    os.makedirs(builddir_)

numCores_ = multiprocessing.cpu_count()

version_ = "3.2.1.dev%s" % stamp

# importlib is available only for python3. Since we build wheels, prefer .so
# extension. This way a wheel built by any python3.x will work with any python3.


class CMakeExtension(Extension):
    def __init__(self, name, **kwargs):
        # don't invoke the original build_ext for this special extension
        import tempfile

        # Create a temp file to create a dummy target. This build raises an
        # exception because sources are empty. With python3 we can fix it by
        # passing `optional=True` to the argument. With python2 there is no
        # getaway from it.
        f = tempfile.NamedTemporaryFile(suffix=".cpp", delete=False)
        f.write(b"int main() { return 1; }")
        Extension.__init__(self, name, sources=[f.name], **kwargs)
        f.close()


class TestCommand(Command):
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        print("[INFO ] Running tests... ")
        os.chdir(builddir_)
        self.spawn(["ctest", "--output-on-failure", "-j%d" % numCores_])
        os.chdir(sdir_)


class build_ext(_build_ext):
    user_options = [
        ("with-boost", None, "Use Boost Libraries (OFF)"),
        ("with-gsl", None, "Use Gnu Scienfific Library (ON)"),
        ("with-gsl-static", None, "Use GNU Scientific Library (static library) (OFF)"),
        ("debug", None, "Build moose in debugging mode (OFF)"),
        ("no-build", None, "DO NOT BUILD. (for debugging/development)"),
    ] + _build_ext.user_options

    def initialize_options(self):
        # Initialize options.
        self.with_boost = 0
        self.with_gsl = 1
        self.with_gsl_static = 0
        self.debug = 0
        self.no_build = 0
        self.cmake_options = {}
        #  super().initialize_options()
        _build_ext.initialize_options(self)

    def finalize_options(self):
        # Finalize options.
        #  super().finalize_options()
        _build_ext.finalize_options(self)
        self.cmake_options["PYTHON_EXECUTABLE"] = os.path.realpath(sys.executable)
        self.cmake_options["VERSION_MOOSE"] = version_
        if self.with_boost:
            self.cmake_options["WITH_BOOST"] = "ON"
            self.cmake_options["WITH_GSL"] = "OFF"
        else:
            if self.with_gsl_static:
                self.cmake_options["GSL_USE_STATIC_LIBRARIES"] = "ON"
        if self.debug:
            self.cmake_options["CMAKE_BUILD_TYPE"] = "Debug"
        else:
            self.cmake_options["CMAKE_BUILD_TYPE"] = "Release"

    def run(self):
        if self.no_build:
            return
        for ext in self.extensions:
            self.build_cmake(ext)
        #  super().run()
        _build_ext.run(self)

    def build_cmake(self, ext):
        global numCores_
        global sdir_
        print("\n==========================================================\n")
        print("[INFO ] Building pymoose in %s ..." % builddir_)
        cmake_args = []
        for k, v in self.cmake_options.items():
            cmake_args.append("-D%s=%s" % (k, v))
        os.chdir(str(builddir_))
        self.spawn(["cmake", str(sdir_)] + cmake_args)
        if not self.dry_run:
            self.spawn(["make", "-j%d" % numCores_])
        os.chdir(str(sdir_))


with open(os.path.join(sdir_, "README.md")) as f:
    readme = f.read()

setup(
    name="pymoose",
    version=version_,
    description="Python scripting interface of MOOSE Simulator (https://moose.ncbs.res.in)",
    long_description=readme,
    long_description_content_type="text/markdown",
    author="MOOSERes",
    author_email="bhalla@ncbs.res.in",
    maintainer="Dilawar Singh",
    maintainer_email="dilawars@ncbs.res.in",
    url="http://moose.ncbs.res.in",
    packages=[
        "rdesigneur",
        "moose",
        "moose.SBML",
        "moose.genesis",
        "moose.neuroml",
        "moose.neuroml2",
        "moose.chemUtil",
        "moose.chemMerge",
    ],
    package_dir={
        "rdesigneur": os.path.join(sdir_, "python", "rdesigneur"),
        "moose": os.path.join(sdir_, "python", "moose"),
    },
    package_data={
        "moose": [
            "_moose.so",
            os.path.join("neuroml2", "schema", "NeuroMLCoreDimensions.xml"),
            os.path.join("chemUtil", "rainbow2.pkl"),
        ]
    },
    # python2 specific version here as well.
    install_requires=["numpy", "matplotlib"],
    extra_requires={"dev": ["coverage", "pytest", "pytest-cov"]},
    ext_modules=[CMakeExtension("dummy", optional=True)],
    cmdclass={"build_ext": build_ext, "test": TestCommand},
)



# -*- coding: utf-8 -*-
'''
*******************************************************************
 * File:            validation.py
 * Description:
 * Author:          HarshaRani
 * E-mail:          hrani@ncbs.res.in
 ********************************************************************/

/**********************************************************************
** This program is part of 'MOOSE', the
** Messaging Object Oriented Simulation Environment,
** also known as GENESIS 3 base code.
**           copyright (C) 2003-2017 Upinder S. Bhalla. and NCBS
Created : Thu May 12 10:19:00 2016(+0530)
Version
Last-Updated: Fri Jul 28 15:50:00 2017(+0530)
          By:
**********************************************************************/

**********************************************************************/
2019
Jan 19: - returned errorMsg
'''

foundLibSBML_ = False
try:
    from libsbml import *
    foundLibSBML_ = True
except Exception as e:
    pass

def validateModel(sbmlDoc):
    if sbmlDoc.getNumErrors() > 0:
        tobecontinued = False
        validError = ""
        for i in range(0,sbmlDoc.getNumErrors()):
            validError = validError+sbmlDoc.getError(i).getMessage()
        #print (validError)
        return False, validError

    if (not sbmlDoc):
        print("validateModel: given a null SBML Document")
        return False, "validateModel: given a null SBML Document"

    consistencyMessages = ""
    validationMessages = ""
    noProblems = True
    numCheckFailures = 0
    numConsistencyErrors = 0
    numConsistencyWarnings = 0
    numValidationErrors = 0
    numValidationWarnings = 0
    # Once the whole model is done and before it gets written out,
    # it's important to check that the whole model is in fact complete,
    # consistent and valid.
    numCheckFailures = sbmlDoc.checkInternalConsistency()
    if (numCheckFailures > 0):
        for i in range(0, numCheckFailures):
            sbmlErr = sbmlDoc.getError(i)
            if (sbmlErr.isFatal() or sbmlErr.isError()):
                noProblems = False
                numConsistencyErrors += 1
            else:
                numConsistencyWarnings += 1
        constStr = sbmlDoc.printErrors()
        if sbmlDoc.printErrors():
            consistencyMessages = constStr

    # If the internal checks fail, it makes little sense to attempt
    # further validation, because the model may be too compromised to
    # be properly interpreted.

    if (numConsistencyErrors > 0):
        consistencyMessages += "Further validation aborted."
    else:
        numCheckFailures = sbmlDoc.checkConsistency()
        validationMessages;
        #numCheckFailures = sbmlDoc.checkL3v1Compatibility()
        if (numCheckFailures > 0):
            for i in range(0, (numCheckFailures)):
                consistencyMessages = sbmlDoc.getErrorLog().toString()
                sbmlErr = sbmlDoc.getError(i)
                if (sbmlErr.isFatal() or sbmlErr.isError()):
                    noProblems = False
                    numValidationErrors += 1
                else:
                    numValidationWarnings += 1

        warning = sbmlDoc.getErrorLog().toString()
        oss = sbmlDoc.printErrors()
        validationMessages = oss

    if (noProblems):
        return True,""
    else:
        if consistencyMessages is None:
            consistencyMessages = ""
        if consistencyMessages != "":
            print("consistency Warning: " + consistencyMessages)

        if (numConsistencyErrors > 0):
            print("ERROR: encountered " +str(numConsistencyErrors) +" consistency error in model " +sbmlDoc.getModel().getId() +"'.")

    if (numConsistencyWarnings > 0):
        print("Notice: encountered " +str(numConsistencyWarnings) +" consistency warning in model " +sbmlDoc.getModel().getId() +"'.")

    if (numValidationErrors > 0):
        print("ERROR: encountered " + str(numValidationErrors) + " validation error in model " +sbmlDoc.getModel().getId() + "'.")
        validationMessages = consistencyMessages
        if (numValidationWarnings > 0):
            print("Notice: encountered " +str(numValidationWarnings) +" validation warning in model " +sbmlDoc.getModel().getId() +"'.")
            validationMessages = consistencyMessages
            
    if validationMessages:
        print(validationMessages)

    return False,validationMessages
    # return ( numConsistencyErrors == 0 and numValidationErrors == 0,
    # consistencyMessages)

if __name__ == '__main__':
    import libsbml
    sbmlDoc = libsbml.readSBML('00001-sbml-l3v1.xml')
    validateModel(sbmlDoc)



# Library of simple models.
import moose

def simple_model_a():
    compt = moose.CubeMesh( '/compt' )
    r = moose.Reac( '/compt/r' )
    a = moose.Pool( '/compt/a' )
    a.concInit = 1
    b = moose.Pool( '/compt/b' )
    b.concInit = 2
    c = moose.Pool( '/compt/c' )
    c.concInit = 0.5
    moose.connect( r, 'sub', a, 'reac' )
    moose.connect( r, 'prd', b, 'reac' )
    moose.connect( r, 'prd', c, 'reac' )
    r.Kf = 0.1
    r.Kb = 0.01

    tabA = moose.Table2( '/compt/a/tab' )
    tabB = moose.Table2( '/compt/tabB' )
    tabC = moose.Table2( '/compt/tabB/tabC' )
    print(tabA, tabB, tabC)
    moose.connect( tabA, 'requestOut', a, 'getConc' )
    moose.connect( tabB, 'requestOut', b, 'getConc' )
    moose.connect( tabC, 'requestOut', c, 'getConc' )
    return [tabA, tabB, tabC]



# -*- coding: utf-8 -*-
from __future__ import print_function, division

## Makes and plots the dose response curve for bistable models
## Author: Sahil Moza
## June 26, 2014
## Update:
## Friday 14 September 2018 05:48:42 PM IST
## Tunrned into a light-weight test by Dilawar Singh 

import os
import sys
import moose
import numpy as np

sdir_ = os.path.dirname( os.path.realpath( __file__ ) )
vals_ = [ ]

def setupSteadyState(simdt,plotDt):

    ksolve = moose.Ksolve( '/model/kinetics/ksolve' )
    stoich = moose.Stoich( '/model/kinetics/stoich' )
    stoich.compartment = moose.element('/model/kinetics')
    stoich.ksolve = ksolve
    stoich.path = "/model/kinetics/##"
    state = moose.SteadyState( '/model/kinetics/state' )
    moose.reinit()
    state.stoich = stoich
    state.showMatrices()
    state.convergenceCriterion = 1e-8
    return ksolve, state

def parseModelName(fileName):
    pos1=fileName.rfind('/')
    pos2=fileName.rfind('.')
    directory=fileName[:pos1]
    prefix=fileName[pos1+1:pos2]
    suffix=fileName[pos2+1:len(fileName)]
    return directory, prefix, suffix

# Solve for the steady state
def getState( ksolve, state, vol):
    scale = 1.0 / ( vol * 6.022e23 )
    moose.reinit()
    state.randomInit() # Removing random initial condition to systematically make Dose reponse curves.
    moose.start( 2.0 ) # Run the model for 2 seconds.
    a = moose.element( '/model/kinetics/a' ).conc
    vals_.append( a )
    state.settle()
    
    vector = []
    a = moose.element( '/model/kinetics/a' ).conc
    for x in ksolve.nVec[0]:
        vector.append( x * scale)
    failedSteadyState = any([np.isnan(x) for x in vector])
    if not (failedSteadyState):
        return state.stateType, state.solutionStatus, a, vector


def test_dose_response():
    # Setup parameters for simulation and plotting
    simdt= 1e-2
    plotDt= 1

    # Factors to change in the dose concentration in log scale
    factorExponent = 10  ## Base: ten raised to some power.
    factorBegin = -10
    factorEnd = 11
    factorStepsize = 2
    factorScale = 10.0 ## To scale up or down the factors

    # Load Model and set up the steady state solver.
    # model = sys.argv[1] # To load model from a file.
    model = os.path.join(sdir_, '..', 'data', '19085.cspace')
    modelPath, modelName, modelType = parseModelName(model)
    outputDir = modelPath
    
    modelId = moose.loadModel(model, 'model', 'ee')
    dosePath = '/model/kinetics/b/DabX' # The dose entity

    ksolve, state = setupSteadyState(simdt, plotDt)
    vol = moose.element('/model/kinetics' ).volume
    iterInit = 100
    solutionVector = []
    factorArr = []
    
    enz = moose.element(dosePath)
    init = float(enz.kcat) # Dose parameter
    
    # Change Dose here to .
    for factor in range(factorBegin, factorEnd, factorStepsize ):
        scale = factorExponent ** (factor/factorScale) 
        enz.kcat = init * scale     
        print( "scale={:.3f}\tkcat={:.3f}".format( scale, enz.kcat) )
        for num in range(iterInit):
            stateType, solStatus, a, vector = getState( ksolve, state, vol)
            if solStatus == 0:
                #solutionVector.append(vector[0]/sum(vector))
                solutionVector.append(a)
                factorArr.append(scale)   

    expected = (0.001411, 0.00092559)
    got = ( np.mean(vals_), np.std(vals_) )
    assert np.isclose(got, expected, atol=1e-4).all(), "Got %s, expected %s" % (got, expected)
    print( '[TEST1] Passed. Concentration of a is same' )
                
    joint = np.array([factorArr, solutionVector])
    joint = joint[:,joint[1,:].argsort()]
    got = np.mean( joint ), np.std( joint )
    expected = (1.2247, 2.46)
    # Close upto 2 decimal place is good enough.
    assert np.isclose(got, expected, atol=1e-2).all(), "Got %s, expected %s" % (got, expected)
    print( joint )

def main():
    test_dose_response()

if __name__ == '__main__':
    main()



# pyrun.py ---
# Author: Subhasis Ray
# Maintainer: Dilawar Singh

# See https://github.com/BhallaLab/moose-examples/blob/b2e77237ef36e47d0080cc8c4fb6bb94313d2b5e/snippets/pyrun.py
# for details.

import os
import moose
import pytest
import sys
import io
import difflib

stdout_ = sys.stdout
if sys.version_info.major > 2:
    stream_ = io.StringIO()
else:
    stream_ = io.BytesIO()
sys.stdout = stream_

# Removed first 3 lines since they change during each run.
expected = """Running Hello
Hello count = 0
Init World
Running World
World count = 0
Running Hello
Hello count = 1
Running World
World count = 1
Running Hello
Hello count = 2
Running World
World count = 2
Running Hello
Hello count = 3
Running World
World count = 3
Running Hello
Hello count = 4
Running World
World count = 4
Running Hello
Hello count = 5
Running World
World count = 5
Running Hello
Hello count = 6
Running World
World count = 6
Running Hello
Hello count = 7
Running World
World count = 7
Running Hello
Hello count = 8
Running World
World count = 8
Running Hello
Hello count = 9
Running World
World count = 9
Running Hello
Hello count = 10
Running World
World count = 10
Running Hello
Hello count = 11
Running World
World count = 11
Running Hello
Hello count = 12
Running World
World count = 12
Running Hello
Hello count = 13
Running World
World count = 13
Running Hello
Hello count = 14
Running World
World count = 14
Running Hello
Hello count = 15
Running World
World count = 15
Running Hello
Hello count = 16
Running World
World count = 16
Running Hello
Hello count = 17
Running World
World count = 17
Running Hello
Hello count = 18
Running World
World count = 18
Running Hello
Hello count = 19
Running World
World count = 19
Running Hello
Hello count = 20
Running World
World count = 20
Init World
Running World
World count = 0
input = 0.0
output = 0.0
Running World
World count = 1
input = 0.0
output = 0.0
Running World
World count = 2
input = 0.0
output = 0.0
Running World
World count = 3
input = 0.0
output = 0.0
Running World
World count = 4
input = 1.0
output = 1.0
Running World
World count = 5
input = 1.0
output = 1.0
Running World
World count = 6
input = 1.0
output = 1.0
Running World
World count = 7
input = 1.0
output = 1.0
Running World
World count = 8
input = 2.0
output = 4.0
Running World
World count = 9
input = 2.0
output = 4.0
Running World
World count = 10
input = 2.0
output = 4.0
Running World
World count = 11
input = 2.0
output = 4.0
Running World
World count = 12
input = 3.0
output = 9.0
Running World
World count = 13
input = 3.0
output = 9.0
Running World
World count = 14
input = 3.0
output = 9.0
Running World
World count = 15
input = 3.0
output = 9.0
Running World
World count = 16
input = 0.0
output = 0.0
Running World
World count = 17
input = 0.0
output = 0.0
Running World
World count = 18
input = 0.0
output = 0.0
Running World
World count = 19
input = 0.0
output = 0.0
Running World
World count = 20
input = 1.0
output = 1.0
Running World
World count = 21
input = 1.0
output = 1.0
Running World
World count = 22
input = 1.0
output = 1.0
Running World
World count = 23
input = 1.0
output = 1.0
Running World
World count = 24
input = 2.0
output = 4.0
Running World
World count = 25
input = 2.0
output = 4.0
Running World
World count = 26
input = 2.0
output = 4.0
Running World
World count = 27
input = 2.0
output = 4.0
Running World
World count = 28
input = 3.0
output = 9.0
Running World
World count = 29
input = 3.0
output = 9.0
Running World
World count = 30
input = 3.0
output = 9.0
Running World
World count = 31
input = 3.0
output = 9.0
Running World
World count = 32
input = 0.0
output = 0.0
Running World
World count = 33
input = 0.0
output = 0.0
Running World
World count = 34
input = 0.0
output = 0.0
Running World
World count = 35
input = 0.0
output = 0.0
Running World
World count = 36
input = 1.0
output = 1.0
Running World
World count = 37
input = 1.0
output = 1.0
Running World
World count = 38
input = 1.0
output = 1.0
Running World
World count = 39
input = 2.0
output = 4.0
Running World
World count = 40
"""

def run_sequence():
    model = moose.Neutral('/model')
    hello_runner = moose.PyRun('/model/Hello')
    hello_runner.initString = """
print( 'Init', moose.element('/model/Hello') )
hello_count = 0
"""
    hello_runner.runString = """
print( 'Running Hello' )
print( 'Hello count =', hello_count )
hello_count += 1
"""
    hello_runner.run('from datetime import datetime')
    hello_runner.run('print("Hello: current time:", datetime.now().isoformat())')
    moose.useClock(0, hello_runner.path, 'process')
    world_runner = moose.PyRun('World')
    world_runner.initString = """
print( 'Init World' )
world_count = 0
def incr_count():
    global world_count
    world_count += 1
"""
    world_runner.runString = """
print( 'Running World' )
print( 'World count =', world_count )
incr_count()
"""
    world_runner.run('from datetime import datetime')
    world_runner.run('print( "World: current time:", datetime.now().isoformat())')

    moose.useClock(0, world_runner.path, 'process')
    moose.reinit()
    moose.start(0.001)


def input_output():
    model = moose.Neutral('/model')
    input_pulse = moose.PulseGen('/model/pulse')
    #: set the baseline output 0
    input_pulse.baseLevel = 0.0
    #: We make it generate three pulses
    input_pulse.count = 3
    input_pulse.level[0] = 1.0
    input_pulse.level[1] = 2.0
    input_pulse.level[2] = 3.0
    #: Each pulse will appear 1 s after the previous one
    input_pulse.delay[0] = 1.0
    input_pulse.delay[1] = 1.0
    input_pulse.delay[2] = 1.0
    #: Each pulse is 1 s wide
    input_pulse.width[0] = 1.0
    input_pulse.width[1] = 1.0
    input_pulse.width[2] = 1.0
    #: Now create the PyRun object
    pyrun = moose.PyRun('/model/pyrun')
    pyrun.runString = """
output = input_ * input_
print( 'input =', input_ )
print( 'output =', output )
"""
    pyrun.mode = 2 # do not run process method
    moose.connect(input_pulse, 'output', pyrun, 'trigger')
    output_table = moose.Table('/model/output')
    moose.connect(pyrun, 'output', output_table, 'input')
    input_table = moose.Table('/model/input')
    moose.connect(input_pulse, 'output', input_table, 'input')
    moose.setClock(0, 0.25)
    moose.setClock(1, 0.25)
    moose.setClock(2, 0.25)
    moose.useClock(0, input_pulse.path, 'process')
    #: this is unnecessary because the mode=2 ensures that `process`
    #: does nothing
    moose.useClock(1, pyrun.path, 'process')
    moose.useClock(2, '/model/#[ISA=Table]', 'process')
    moose.reinit()
    moose.start(10.0)


# This test will not pass with doctest and coverage
@pytest.mark.xfail(reason="Would not pass with python-coverage")
def test_pyrun():
    global stream_, stdout_, expected
    run_sequence()
    moose.delete('/model')
    input_output()
    sys.stdout = stdout_
    stream_.flush()
    sys.stdout.flush()
    expected = expected.split('\n')[-30:]
    got = stream_.getvalue().split('\n')[-30:]

    if os.environ.get('TRAVIS_OS_NAME', '') != 'osx':
        for x, y in zip(expected, got):
            print("{0:40s} {1:}".format(x, y))
        # Deleted first 3 lines.
        try:
            assert expected == got
        except Exception:
            s = difflib.SequenceMatcher(None, '\n'.join(expected), '\n'.join(got))
            assert s.ratio() >= 0.70, ("Difference is too large", s.ratio())
        print('All done')
    else:
        print("Allow failure on Travis/OSX but not locally.")


if __name__ == '__main__':
    test_pyrun()



#########################################################################
## This program is part of 'MOOSE', the
## Messaging Object Oriented Simulation Environment.
##           Copyright (C) 2014 Upinder S. Bhalla. and NCBS
## It is made available under the terms of the
## GNU Lesser General Public License version 2.1
## See the file COPYING.LIB for the full notice.
#########################################################################


import sys
import math
import numpy as np
import moose
print('[INFO] Using moose from %s' % moose.__file__ )
import os

moose.seed( 1 )

concA = 0.005 # millimolar
def makeModel():
    # create container for model
    r0 = 2e-6        # m
    r1 = 1e-6        # m
    num = 100
    diffLength = 1e-6 # m
    L = num * diffLength        # m
    diffConst = 10e-12
    #motorRate = 1e-6
    #diffConst = 0
    motorRate = 0

    model = moose.Neutral( 'model' )
    compartment = moose.CylMesh( '/model/compartment' )
    compartment.r0 = r0
    compartment.r1 = r1
    compartment.x0 = 0
    compartment.x1 = L
    compartment.diffLength = diffLength
    assert( compartment.numDiffCompts == num )

    # create molecules and reactions
    a = moose.Pool( '/model/compartment/a' )
    b = moose.Pool( '/model/compartment/b' )
    c = moose.Pool( '/model/compartment/c' )
    d = moose.Pool( '/model/compartment/d' )
    r1 = moose.Reac( '/model/compartment/r1' )
    moose.connect( r1, 'sub', b, 'reac' )
    moose.connect( r1, 'sub', d, 'reac' )
    moose.connect( r1, 'prd', c, 'reac' )
    r1.Kf = 1000.0 # 1/(mM.sec)
    r1.Kb = 1 # 1/sec

    # Assign parameters
    a.diffConst = diffConst
    b.diffConst = diffConst / 2.0
    b.motorConst = motorRate
    c.diffConst = diffConst
    d.diffConst = diffConst

    # Make solvers
    ksolve = moose.Gsolve( '/model/compartment/ksolve' )
    dsolve = moose.Dsolve( '/model/compartment/dsolve' )
    stoich = moose.Stoich( '/model/compartment/stoich' )
    stoich.compartment = compartment
    stoich.ksolve = ksolve
    stoich.dsolve = dsolve
    stoich.path = "/model/compartment/##"
    assert( dsolve.numPools == 4 )
    a.vec.concInit = concA
    b.vec.concInit = concA / 5.0
    c.vec.concInit = concA
    d.vec.concInit = concA / 5.0
    for i in range( num ):
        d.vec[i].concInit = concA * 2 * i / num

def test_diffusion_gsolve_dsolve():
    """
    This example illustrates how to set up a diffusion/transport model with
    a simple reaction-diffusion system in a tapering cylinder:

    | Molecule **a** diffuses with diffConst of 10e-12 m^2/s.
    | Molecule **b** diffuses with diffConst of 5e-12 m^2/s.
    | Molecule **b** also undergoes motor transport with a rate of 10e-6 m/s
    |   Thus it 'piles up' at the end of the cylinder.
    | Molecule **c** does not move: diffConst = 0.0
    | Molecule **d** does not move: diffConst = 10.0e-12 but it is buffered.
    |   Because it is buffered, it is treated as non-diffusing.

    All molecules other than **d** start out only in the leftmost (first)
    voxel, with a concentration of 1 mM. **d** is present throughout
    at 0.2 mM, except in the last voxel, where it is at 1.0 mM.

    The cylinder has a starting radius of 2 microns, and end radius of
    1 micron. So when the molecule undergoing motor transport gets to the
    narrower end, its concentration goes up.

    There is a little reaction in all compartments: ``b + d <===> c``

    As there is a high concentration of **d** in the last compartment,
    when the molecule **b** reaches the end of the cylinder, the reaction
    produces lots of **c**.

    Note that molecule **a** does not participate in this reaction.

    The concentrations of all molecules are displayed in an animation.
    """
    runtime = 20.0
    diffdt = 0.005
    plotdt = 0.1
    makeModel()
    # Set up clocks. The dsolver to know before assigning stoich
    moose.setClock( 10, diffdt ) # 10 is the standard clock for Dsolve.
    moose.setClock( 16, plotdt ) # 16 is the standard clock for Ksolve.

    a = moose.element( '/model/compartment/a' )
    b = moose.element( '/model/compartment/b' )
    c = moose.element( '/model/compartment/c' )
    d = moose.element( '/model/compartment/d' )

    moose.reinit()
    atot = sum( a.vec.n )
    btot = sum( b.vec.n )
    ctot = sum( c.vec.n )
    dtot = sum( d.vec.n )
    for t in np.arange( 0, runtime, plotdt ):
        moose.start( plotdt )

    atot2 = sum( a.vec.n )
    btot2 = sum( b.vec.n )
    ctot2 = sum( c.vec.n )
    dtot2 = sum( d.vec.n )

    msg = 'Ratio of initial to final total numbers of '
    got = np.array((atot2/atot, btot2/btot, ctot2/ctot, dtot2/dtot))
    msg += 'a=%f b=%f, c=%f, d=%f' % (tuple(got))
    print(msg)
    print('Initial to final (b+c)=%f' % (float(btot2 + ctot2) / (btot + ctot )))
    expected = np.array((1.00003087, 1.39036644, 0.92191184, 1.11427514))
    error = got - expected
    rerror = np.abs( error ) / expected
    assert np.allclose(got, expected, atol=1e-3), "Got %s, expected %s" % (got, expected)

def main():
    test_diffusion_gsolve_dsolve()

# Run the 'main' if this script is executed standalone.
if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
"""test_muparser.py:

odified from https://elifesciences.org/articles/25827 
Fig4.py
"""
import sys
import re
import numpy as np
import moose
import rdesigneur as rd
print( "[INFO ] Using moose from %s" % moose.__file__ )

def parseExpr( expr, params, hasCa ):
    if hasCa:
        expr = expr.replace( 'Ca', 'x0' )
        expr = expr.replace( 'A', 'x1' )
        expr = expr.replace( 'B', 'x2' )
    else:
        expr = expr.replace( 'Ca', 'x0' ) # happens in the negFF model
        expr = expr.replace( 'A', 'x0' ) # This is the usual case.
        expr = expr.replace( 'B', 'x1' )

    parts = re.split( 'k', expr )
    ret = parts[0]
    for i in parts[1:]:
        ret += str( params[ 'k' + i[:2] ] )
        ret += i[2:]

    if hasCa:
        return 'x3*( ' + ret + ')'
    else:
        return 'x2*( ' + ret + ')'

def makeChemProto( name, Aexpr, Bexpr, params ):
    sw = params['stimWidth']
    diffLength = params['diffusionLength']
    dca = params['diffConstA'] * diffLength * diffLength
    dcb = params['diffConstB'] * diffLength * diffLength

    # Objects
    chem = moose.Neutral( '/library/' + name )
    compt = moose.CubeMesh( '/library/' + name + '/' + name )
    A = moose.Pool( compt.path + '/A' )
    B = moose.Pool( compt.path + '/B' )
    Z = moose.BufPool( compt.path + '/Z' )
    Ca = moose.BufPool( compt.path + '/Ca' )
    phase = moose.BufPool( compt.path + '/phase' )
    vel = moose.BufPool( compt.path + '/vel' )
    ampl = moose.BufPool( compt.path + '/ampl' )
    Adot = moose.Function( A.path + '/Adot' )
    Bdot = moose.Function( B.path + '/Bdot' )
    CaStim = moose.Function( Ca.path + '/CaStim' )
    A.diffConst = dca
    B.diffConst = dcb

    # Equations

    Adot.expr = parseExpr( Aexpr, params, True )
    Bdot.expr = parseExpr( Bexpr, params, False )
    CaStim.expr = 'x2 * exp( -((x0 - t)^2)/(2* ' + str(sw*sw) + ') )'

    #print Adot.expr
    #print Bdot.expr
    #print CaStim.expr

    # Connections
    Adot.x.num = 4
    moose.connect( Ca, 'nOut', Adot.x[0], 'input' )
    moose.connect( A, 'nOut', Adot.x[1], 'input' )
    moose.connect( B, 'nOut', Adot.x[2], 'input' )
    moose.connect( Z, 'nOut', Adot.x[3], 'input' )
    moose.connect( Adot, 'valueOut', A, 'increment' )

    Bdot.x.num = 3
    if name[:5] == 'negFF':
        moose.connect( Ca, 'nOut', Bdot.x[0], 'input' )
        print('Doing special msg')
    else:
        moose.connect( A, 'nOut', Bdot.x[0], 'input' )
    moose.connect( B, 'nOut', Bdot.x[1], 'input' )
    moose.connect( Z, 'nOut', Bdot.x[2], 'input' )
    moose.connect( Bdot, 'valueOut', B, 'increment' )

    CaStim.x.num = 3
    moose.connect( phase, 'nOut', CaStim.x[0], 'input' )
    moose.connect( vel, 'nOut', CaStim.x[1], 'input' )
    moose.connect( ampl, 'nOut', CaStim.x[2], 'input' )
    moose.connect( CaStim, 'valueOut', Ca, 'setN' )

    return compt


def makeBis( args = None ):
    params = {
        'k0a':0.1,  # Constant
        'k1a':-5.0,  # Coeff for A
        'k2a':5.0,  # Coeff for A^2
        'k3a':-1.0,  # Coeff for A^3
        'k4a':10.0,  # turnon of A by A and Ca
        'k5a':-5.0,  # Turnoff of A by B
        'k1b':0.01,  # turnon of B by A
        'k2b':-0.01,   # Decay rate of B
        'diffusionLength':1.0e-6,  # Diffusion characteristic length, used as voxel length too.
        'dendDiameter': 10e-6,  # Diameter of section of dendrite in model
        'dendLength': 100e-6,   # Length of section of dendrite in model
        'diffConstA':5.0,       # Diffusion constant of A
        'diffConstB':2.0,       # Diffusion constant of B
        'stimWidth' :1.0,        # Stimulus width in seconds
        'stimAmplitude':1.0,    # Stimulus amplitude, arb units. From FHN review
        'blankVoxelsAtEnd':10,  # of voxels to leave blank at end of cylinder
        'preStimTime':10.0,     # Time to run before turning on stimulus.
        'postStimTime':40.0,    # Time to run after stimulus. ~3x decay time
        'settleTime':20.0,    # Settling time of response, after stimulus.
                              # To include in analysis of total response over
                              # entire dendrite.
        'fnumber':1,          # Number to append to fname
    }

    for i in args:
        params[i] = args[i]

    makeChemProto( 'bis',
        'k0a + k1a*A + k2a*A*A + k3a*A*A*A + k4a*Ca*A/(1+A+10*B) + k5a*A*B',
        'k1b*A*A + k2b*B',
        params )
    return params

def makeFHN( args = None ):
    params = {
        'k_t':2.5,  # Time-const.
        'k_a':0.7,  # Coeff1
        'k_b':0.8,  # Coeff2
        'kxa': 2.0, # Offset for V for FHN eqns.
        'kxb': 0.8, # Offset for W for FHN eqns.
        'diffusionLength':1.0e-6,  # Diffusion characteristic length, used as voxel length too.
        'dendDiameter': 10e-6,  # Diameter of section of dendrite in model
        'dendLength': 100e-6,   # Length of section of dendrite in model
        'diffConstA':7.5,      # Diffusion constant of A
        'diffConstB':5.0,       # Diffusion constant of B
        'stimWidth' :1.0,        # Stimulus width in seconds
        'stimAmplitude':0.4,    # Stimulus amplitude, arb units. From FHN review
        'blankVoxelsAtEnd':10,  # of voxels to leave blank at end of cylinder
        'preStimTime':10.0,     # Time to run before turning on stimulus.
        'postStimTime':40.0,    # Time to run after stimulus. ~3x decay time
        'settleTime':20.0,    # Settling time of response, after stimulus.
                              # To include in analysis of total response over
                              # entire dendrite.
        'fnumber': 1,         # Number to append to fname
    }

    for i in args:
        params[i] = args[i]

    makeChemProto( 'fhn',
        '5.0*(A - kxa - (A-kxa)^3/3 - (B-kxb) + Ca)',
        '(A-kxa + k_a - k_b*(B-kxb))/k_t',
        params )
    # We do this to get the system to settle at the start.
    B = moose.element( '/library/fhn/fhn/B' )
    B.nInit = 0.2
    return params


def makeNegFB( args = None ):
    params = {
        'k1a':-0.1,  # Coeff for decay of A, slow.
        'k2a':-0.2,  # Coeff for turnoff of A by B, medium.
        'k3a':10.0,  # turnon of A by Ca, fast.
        'k1b':0.2,  # turnon of B by A, medium
        'k2b':-0.1,   # Decay rate of B, slow
        'diffusionLength':1.0e-6,  # Diffusion characteristic length, used as voxel length too.
        'dendDiameter': 10e-6,  # Diameter of section of dendrite in model
        'dendLength': 100e-6,   # Length of section of dendrite in model
        'diffConstA':0.5,      # Diffusion constant of A
        'diffConstB':1.0,       # Diffusion constant of B
        'stimWidth' :1.0,        # Stimulus width in seconds
        'stimAmplitude':1.0,    # Stimulus amplitude, arb units. From FHN review
        'blankVoxelsAtEnd':10,  #of voxels to leave blank at end of cylinder
        'preStimTime':10.0,     # Time to run before turning on stimulus.
        'postStimTime':40.0,    # Time to run after stimulus. ~3x decay time
        'settleTime':20.0,    # Settling time of response, after stimulus.
                              # To include in analysis of total response over
                              # entire dendrite.
        'fnumber':1,          # Number to append to fname
    }

    for i in args:
        params[i] = args[i]

    makeChemProto( 'negFB',
        'k1a*A + k2a*A*B + k3a*Ca',
        'k1b*A + k2b*B',
        params )
    return params

# Was negFF2 in earlier versions of abstrModelEqns
def makeNegFF( args = None ):
    params = {
        'k1a':-0.1,  # Coeff for decay of A, slow.
        'k2a':-0.01,  # Coeff for turnoff of A by B, medium.
        'k3a':10.0,  # turnon of A by Ca, fast.
        'k4a':40.0,   # amount of B inhibition of turnon of A by Ca.
        'k1b':2.0,  # turnon of B by Ca, medium
        'k2b':-0.05,   # Decay rate of B, slow
        'diffusionLength':1.0e-6,  # Diffusion characteristic length, used as voxel length too.
        'dendDiameter': 10e-6,  # Diameter of section of dendrite in model
        'dendLength': 100e-6,   # Length of section of dendrite in model
        'diffConstA':0.02,      # Diffusion constant of A
        'diffConstB':0.4,       # Diffusion constant of B
        'stimWidth' :1.0,        # Stimulus width in seconds
        'stimAmplitude':10.0,    # Stimulus amplitude, arb units. From FHN review
        'blankVoxelsAtEnd':10,  # of voxels to leave blank at end of cylinder
        'preStimTime':10.0,     # Time to run before turning on stimulus.
        'postStimTime':40.0,    # Time to run after stimulus. ~3x decay time
        'settleTime':20.0,    # Settling time of response, after stimulus.
                              # To include in analysis of total response over
                              # entire dendrite.
        'fnumber':1,          # Number to append to fname
    }

    for i in args:
        params[i] = args[i]

    makeChemProto( 'negFF',
        'k1a*A + k2a*A*B + k3a*Ca/(1+k4a*B*B)',
        'k1b*Ca + k2b*B',
        params )

    return params


def singleCompt( name, params ):
    print('=============')
    print('[INFO] Making compartment %s' % name)
    mod = moose.copy( '/library/' + name + '/' + name, '/model' )
    A = moose.element( mod.path + '/A' )
    Z = moose.element( mod.path + '/Z' )
    Z.nInit = 1
    Ca = moose.element( mod.path + '/Ca' )
    CaStim = moose.element( Ca.path + '/CaStim' )

    print( '\n\n[INFO] CaStim %s' % CaStim.path )
    runtime = params['preStimTime'] + params['postStimTime']
    steptime = 50
    CaStim.expr += '+x2*(t>100+'+str(runtime)+')*(t<100+'+str(runtime+steptime)+ ')'
    print("[INFO] CaStim.expr = %s" % CaStim.expr)
    tab = moose.Table2( '/model/' + name + '/Atab' )
    ampl = moose.element( mod.path + '/ampl' )
    phase = moose.element( mod.path + '/phase' )
    moose.connect( tab, 'requestOut', A, 'getN' )
    ampl.nInit = params['stimAmplitude'] * 1
    phase.nInit = params['preStimTime']

    ksolve = moose.Ksolve( mod.path + '/ksolve' )
    stoich = moose.Stoich( mod.path + '/stoich' )
    stoich.compartment = mod
    stoich.ksolve = ksolve
    stoich.path = mod.path + '/##'

    print( 'REINIT AND START' )
    moose.reinit()
    runtime += 100 + steptime*2
    moose.start( runtime )
    t = np.arange( 0, runtime + 1e-9, tab.dt )
    return name, t, tab.vector


def runPanelDEFG( name, dist, seqDt, numSpine, seq, stimAmpl ):
    preStim = 10.0
    blanks = 20
    rdes = rd.rdesigneur(
        useGssa = False,
        turnOffElec = True,
        chemPlotDt = 0.1,
        diffusionLength = 1e-6,
        cellProto = [['cell', 'soma']],
        chemProto = [['dend', name]],
        chemDistrib = [['dend', 'soma', 'install', '1' ]],
        plotList = [['soma', '1', 'dend' + '/A', 'n', '# of A']],
    )
    rdes.buildModel()
    A = moose.vec( '/model/chem/dend/A' )
    Z = moose.vec( '/model/chem/dend/Z' )
    print(moose.element( '/model/chem/dend/A/Adot' ).expr)
    print(moose.element( '/model/chem/dend/B/Bdot' ).expr)
    print(moose.element( '/model/chem/dend/Ca/CaStim' ).expr)
    phase = moose.vec( '/model/chem/dend/phase' )
    ampl = moose.vec( '/model/chem/dend/ampl' )
    vel = moose.vec( '/model/chem/dend/vel' )
    vel.nInit = 1e-6 * seqDt
    ampl.nInit = stimAmpl
    stride = int( dist ) / numSpine
    phase.nInit = 10000
    Z.nInit = 0
    for j in range( numSpine ):
        k = int( blanks + j * stride )
        Z[k].nInit = 1
        phase[k].nInit = preStim + seq[j] * seqDt

    moose.reinit()
    runtime = 50
    snapshot = preStim + seqDt * (numSpine - 0.8)
    print(snapshot)
    moose.start( snapshot )
    avec = moose.vec( '/model/chem/dend/A' ).n
    moose.start( runtime - snapshot )
    tvec = []
    for i in range( 5 ):
        tab = moose.element( '/model/graphs/plot0[' + str( blanks + i * stride ) + ']' )
        dt = tab.dt
        tvec.append( tab.vector )
    moose.delete( '/model' )
    return dt, tvec, avec

def makePassiveSoma( name, length, diameter ):
    elecid = moose.Neuron( '/library/' + name )
    dend = moose.Compartment( elecid.path + '/soma' )
    dend.diameter = diameter
    dend.length = length
    dend.x = length
    return elecid

def run():
    panelC = []
    panelCticks = []
    panelC.append( singleCompt( 'negFB', makeNegFB( [] ) ) )
    panelC.append( singleCompt( 'negFF', makeNegFF( [] ) ) )
    panelC.append( singleCompt( 'fhn', makeFHN( [] ) ) )
    panelC.append( singleCompt( 'bis', makeBis( [] ) ) )
    panelCticks.append( np.arange( 0, 15.00001, 5 ) )
    panelCticks.append( np.arange( 0, 1.50001, 0.5 ) )
    panelCticks.append( np.arange( 0, 5.00002, 1 ) )
    panelCticks.append( np.arange( 0, 5.00002, 1 ) )
    moose.delete( '/model' )
    for i in zip( panelC, panelCticks, list(range( len( panelC ))) ):
        plotPos = i[2] + 5
        doty = i[1][-1] * 0.95
        print('doty',  doty )
    print( '[INFO] Run is over')

def test_function_chemsys():
    moose.Neutral( '/library' )
    moose.Neutral( '/model' )
    run()
    print( 'All done' )

def main():
    test_function_chemsys()

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
# test_nsdf.py ---
# Changed from nsdf.py from
# https://github.com/BhallaLab/moose-examples/snippets/nsdf.py

from __future__ import print_function
import numpy as np
from datetime import datetime
import getpass
import os

# Use in-repo moose to install.
import moose
print('using moose from: %s' % moose.__file__)

global nsdf

def setup_model():
    """Setup a dummy model with a PulseGen and a SpikeGen. The SpikeGen
    detects the leading edges of the pulses created by the PulseGen
    and sends out the event times. We record the PulseGen outputValue
    as Uniform data and leading edge time as Event data in the NSDF
    file.

    """
    global nsdf
    simtime = 100.0
    dt = 1e-3
    model = moose.Neutral('/model')
    pulse = moose.PulseGen('/model/pulse')
    pulse.level[0] = 1.0
    pulse.delay[0] = 10
    pulse.width[0] = 20
    t_lead = moose.SpikeGen('/model/t_lead')
    t_lead.threshold = 0.5
    moose.connect(pulse, 'output', t_lead,'Vm');
    nsdf = moose.NSDFWriter('/model/writer')
    nsdf.filename = 'nsdf_demo.h5'
    nsdf.mode = 2 #overwrite existing file
    nsdf.flushLimit = 100
    moose.connect(nsdf, 'requestOut', pulse, 'getOutputValue')
    print('event input', nsdf.eventInput, nsdf.eventInput.num)
    print(nsdf)

    nsdf.eventInput.num = 1
    ei = nsdf.eventInput[0]
    print(ei.path)
    moose.connect(t_lead, 'spikeOut', nsdf.eventInput[0], 'input')
    tab = moose.Table('spiketab')
    tab.threshold = t_lead.threshold
    clock = moose.element('/clock')
    for ii in range(32):
        moose.setClock(ii, dt)
    moose.connect(pulse, 'output', tab, 'spike')
    print(datetime.now().isoformat())
    moose.reinit()
    moose.start(simtime)
    print(datetime.now().isoformat())
    np.savetxt('nsdf.txt', tab.vector)
    ###################################
    # Set the environment attributes
    ###################################
    nsdf.stringAttr['title'] = 'NSDF writing demo for moose'
    nsdf.stringAttr['description'] = '''An example of writing data to NSDF file from MOOSE simulation. In
this simulation we generate square pules from a PulseGen object and
use a SpikeGen to detect the threshold crossing events of rising
edges. We store the pulsegen output as Uniform data and the threshold
crossing times as Event data. '''
    nsdf.stringAttr['creator'] = getpass.getuser()
    nsdf.stringVecAttr['software'] = ['python2.7', 'moose3' ]
    nsdf.stringVecAttr['method'] = ['']
    nsdf.stringAttr['rights'] = ''
    nsdf.stringAttr['license'] = 'CC-BY-NC'
    # Specify units. MOOSE is unit agnostic, so we explicitly set the
    # unit attibutes on individual datasets
    nsdf.stringAttr['/data/uniform/PulseGen/outputValue/tunit'] = 's'
    nsdf.stringAttr['/data/uniform/PulseGen/outputValue/unit'] = 'A'
    eventDataPath = '/data/event/SpikeGen/spikeOut/{}_{}_{}/unit'.format(t_lead.vec.value,
                                                                         t_lead.getDataIndex(),
                                                                         t_lead.fieldIndex)
    nsdf.stringAttr[eventDataPath] = 's'

def test_nsdf():
    try:
        setup_model()
    except AttributeError as e:
        print( 'This MOOSE is not compiled with NSDF support' )
        return 0
    except Exception as e:
        raise e
    # Very basic tests
    nsdfFile = 'nsdf.txt'
    if not os.path.exists( nsdf.filename ):
        raise Exception("Test failed. No files : %s" % nsdfFile)
    if not os.path.exists( nsdfFile ):
        raise Exception("Test failed. No file : %s" % nsdfFile)

    data = np.loadtxt( nsdfFile )
    assert len(data) == 4, "Expected 4 entries"

def main():
    test_nsdf()

if __name__ == '__main__':
    main()
